# django_spellbook/tests/test_url_view_generator.py

import unittest
from unittest.mock import Mock, patch, MagicMock
from pathlib import Path
from django.core.management.base import CommandError
from django.test import TestCase

from django_spellbook.management.commands.processing.url_view_generator import URLViewGenerator
from django_spellbook.markdown.context import SpellbookContext
from django_spellbook.management.commands.processing.file_processor import ProcessedFile


class TestURLViewGenerator(TestCase):
    def setUp(self):
        """Set up the test environment."""
        # Initialize with patches for the components
        with patch('django_spellbook.management.commands.processing.url_view_generator.URLGenerator') as mock_url_gen:
            with patch('django_spellbook.management.commands.processing.url_view_generator.ViewGenerator') as mock_view_gen:
                with patch('django_spellbook.management.commands.processing.url_view_generator.FileWriter') as mock_file_writer:
                    self.generator = URLViewGenerator('test_app', '/test/path', '/source/path')
                    
                    # Store the mocks for later use
                    self.mock_url_gen = mock_url_gen.return_value
                    self.mock_view_gen = mock_view_gen.return_value
                    self.mock_file_writer = mock_file_writer.return_value
        
        # Create a mock context
        self.mock_context = Mock(spec=SpellbookContext)
        self.mock_context.__dict__ = {'title': 'Test', 'toc': {}}

        # Create a sample processed file
        self.processed_file = ProcessedFile(
            original_path=Path('/test/file.md'),
            html_content='<h1>Test</h1>',
            template_path=Path('/test/template.html'),
            relative_url='test/page',
            context=self.mock_context
        )

    def test_initialization(self):
        """Test URLViewGenerator initialization."""
        # Check the stored values
        self.assertEqual(self.generator.content_app, 'test_app')
        self.assertEqual(self.generator.content_dir_path, '/test/path')
        self.assertEqual(self.generator.source_path, '/source/path')
        
        # Check the component initializations
        self.assertIsNotNone(self.generator.url_generator)
        self.assertIsNotNone(self.generator.view_generator)
        self.assertIsNotNone(self.generator.file_writer)

    def test_generate_urls_and_views(self):
        """Test generating URLs and views."""
        # Configure the mocks
        self.mock_url_gen.generate_url_patterns.return_value = ["url_pattern1", "url_pattern2"]
        self.mock_view_gen.generate_view_functions.return_value = ["view_function1", "view_function2"]
        
        # Create a sample TOC
        toc = {"test": {"title": "Test"}}
        
        # Call the method
        self.generator.generate_urls_and_views([self.processed_file], toc)
        
        # Verify the calls to the components
        self.mock_url_gen.generate_url_patterns.assert_called_once_with([self.processed_file])
        self.mock_view_gen.generate_view_functions.assert_called_once_with([self.processed_file])
        self.mock_file_writer.write_urls_file.assert_called_once_with(["url_pattern1", "url_pattern2"])
        self.mock_file_writer.write_views_file.assert_called_once_with(["view_function1", "view_function2"], toc)

    def test_generate_urls_and_views_error(self):
        """Test error handling in generate_urls_and_views."""
        # Configure the mock to raise an error
        self.mock_url_gen.generate_url_patterns.side_effect = Exception("Test error")
        
        # Call the method - should raise CommandError
        with self.assertRaises(CommandError) as context:
            self.generator.generate_urls_and_views([self.processed_file], {})
        
        # Check error message
        self.assertIn("Failed to generate URLs and views", str(context.exception))
        self.assertIn("Test error", str(context.exception))
import os
import shutil
import tempfile
import datetime



class URLViewGeneratorIntegrationTest(TestCase):
    """Integration tests for URL name generation in URLViewGenerator"""
    
    def setUp(self):
        # Create temporary directory for testing
        self.temp_dir = tempfile.mkdtemp()
        
        # Create a temporary spellbook directory structure
        self.spellbook_dir = os.path.join(self.temp_dir, 'django_spellbook')
        os.makedirs(self.spellbook_dir, exist_ok=True)
        
        # Set up patches for the components used by URLViewGenerator
        patcher1 = patch('django_spellbook.management.commands.processing.url_view_generator.URLGenerator')
        patcher2 = patch('django_spellbook.management.commands.processing.url_view_generator.ViewGenerator')
        patcher3 = patch('django_spellbook.management.commands.processing.url_view_generator.FileWriter')
        self.mock_url_gen = patcher1.start()
        self.mock_view_gen = patcher2.start()
        self.mock_file_writer = patcher3.start()
        
        # Get the mock instances
        self.mock_url_gen_instance = self.mock_url_gen.return_value
        self.mock_view_gen_instance = self.mock_view_gen.return_value
        self.mock_file_writer_instance = self.mock_file_writer.return_value
        
        # Configure URL generator mock to return expected URL patterns
        self.expected_url_patterns = [
            "path('first_blog/', first_blog, name='first_blog')",
            "path('lifestyle/digital-minimalism/', lifestyle_digital_minimalism, name='lifestyle_digital-minimalism')",
            "path('blocks/practice/', blocks_practice, name='blocks_practice')",
            "path('blocks/quote/', blocks_quote, name='blocks_quote')",
            "path('tech/sustainable-tech/', tech_sustainable_tech, name='tech_sustainable-tech')"
        ]
        self.mock_url_gen_instance.generate_url_patterns.return_value = self.expected_url_patterns
        
        # Set up the generator
        self.generator = URLViewGenerator('test_app', os.path.join(self.temp_dir, 'content'))
        
        # Add the patchers to cleanup
        self.addCleanup(patcher1.stop)
        self.addCleanup(patcher2.stop)
        self.addCleanup(patcher3.stop)
        
        # Create a mock context for processed files
        self.context = SpellbookContext(
            title='Test',
            created_at=datetime.datetime(2024, 11, 10, 3, 29, 58, 8432),
            updated_at=datetime.datetime(2024, 11, 10, 3, 29, 58, 8432),
            url_path='test',
            raw_content='# Test\nThis is a test',
        )
        
    def tearDown(self):
        # Clean up temporary directory
        shutil.rmtree(self.temp_dir)
    
    def _create_processed_file(self, relative_url):
        """Helper to create a processed file with a specific relative URL"""
        return ProcessedFile(
            original_path=Path(f"/test/{relative_url}.md"),
            html_content="<h1>Test</h1>",
            template_path=Path(f"/test/templates/{relative_url}.html"),
            relative_url=relative_url,
            context=self.context
        )
    
    def test_actual_urls_file_generation(self):
        """Test the actual generation of the urls.py file with proper URL names"""
        # Process multiple files with different path structures
        processed_files = [
            self._create_processed_file("----first_blog"),
            self._create_processed_file("lifestyle/--digital-minimalism"),
            self._create_processed_file("blocks/practice"),
            self._create_processed_file("blocks/-quote"),
            self._create_processed_file("tech/sustainable-tech")
        ]
        
        # Generate URLs and views
        self.generator.generate_urls_and_views(processed_files, {})
        
        # Verify URL generator was called with the processed files
        self.mock_url_gen_instance.generate_url_patterns.assert_called_once_with(processed_files)
        
        # Verify FileWriter.write_urls_file was called with the expected URL patterns
        self.mock_file_writer_instance.write_urls_file.assert_called_once_with(self.expected_url_patterns)
        
        # Verify the expected URL patterns match what we want
        self.assertIn("path('first_blog/', first_blog, name='first_blog')", self.expected_url_patterns)
        self.assertIn("path('lifestyle/digital-minimalism/', lifestyle_digital_minimalism, name='lifestyle_digital-minimalism')", self.expected_url_patterns)
        self.assertIn("path('blocks/practice/', blocks_practice, name='blocks_practice')", self.expected_url_patterns)
        self.assertIn("path('blocks/quote/', blocks_quote, name='blocks_quote')", self.expected_url_patterns)
        self.assertIn("path('tech/sustainable-tech/', tech_sustainable_tech, name='tech_sustainable-tech')", self.expected_url_patterns)


# django_spellbook/management/commands/command_utils.py

import os
import logging
from typing import List, Tuple, Optional
from pathlib import Path

from django.core.management.base import CommandError
from django.conf import settings

logger = logging.getLogger(__name__)

def normalize_settings(setting_path: str, setting_app:str):
    """
    Convert settings to normalized lists with backward compatibility.
    
    Args:
        setting_path (str): Path to markdown files.
        setting_app (str): Name of the content app.
    
    Returns:
        Tuple[List[str], List[str]]: md_paths and md_apps.
    """
    md_paths: List[str] = [setting_path] if isinstance(setting_path, (str, Path)) else setting_path
    md_apps: List[str] = [setting_app] if isinstance(setting_app, str) else setting_app
    return md_paths, md_apps

def validate_spellbook_settings():
    """
    Validate required settings and support multiple source-destination pairs.
    
    Returns:
        Tuple[List[str], List[str]]: Normalized settings.
    """
    # Get settings values with backward compatibility
    md_path = getattr(settings, 'SPELLBOOK_MD_PATH', None)
    md_app = getattr(settings, 'SPELLBOOK_MD_APP', None)
    content_app = getattr(settings, 'SPELLBOOK_CONTENT_APP', None)
    
    # Prefer SPELLBOOK_MD_APP but fall back to SPELLBOOK_CONTENT_APP
    app_setting = md_app if md_app is not None else content_app
    
    if md_app is None and content_app is not None:
        logger.warning("SPELLBOOK_CONTENT_APP is deprecated, use SPELLBOOK_MD_APP instead.")
    
    # Normalize settings to lists
    md_file_paths, content_apps = normalize_settings(md_path, app_setting)
    
    # Validate settings
    _validate_setting_values(md_file_paths, content_apps)
    
    return md_file_paths, content_apps

def _validate_setting_values(md_file_paths: List[str], content_apps: List[str]):
    """
    Validate setting values for correctness and compatibility.
    
    Args:
        md_file_paths (List[str]): List of paths to markdown files.
        content_apps (List[str]): List of content app names.
        
    Raises:
        CommandError: If any settings are missing or invalid.
    """
    # Check for missing settings
    missing_settings = []
    if not md_file_paths:
        missing_settings.append('SPELLBOOK_MD_PATH')
    if not content_apps:
        missing_settings.append('SPELLBOOK_MD_APP or SPELLBOOK_CONTENT_APP')
    
    if missing_settings:
        raise CommandError(f"Missing required settings: {', '.join(missing_settings)}")
    
    # Validate list lengths match
    if len(md_file_paths) != len(content_apps):
        raise CommandError("SPELLBOOK_MD_PATH and SPELLBOOK_MD_APP must have the same number of entries")
    
    # Ensure each string is not empty
    for md_path in md_file_paths:
        if not md_path:
            raise CommandError(
                "Invalid SPELLBOOK_MD_PATH configuration!\n"
                "Reason: Empty value found in path list\n"
                "Solution: Ensure all paths in settings.SPELLBOOK_MD_PATH "
                "are non-empty strings\n"
                "Documentation: https://django-spellbook.org/docs/settings/"
            )
    for app_setting in content_apps:
        if not app_setting:
            raise CommandError("SPELLBOOK_MD_APP must be a non-empty string.")

def setup_directory_structure(content_app: str, dirpath: str):
    """
    Set up the necessary directory structure for content processing.
    
    Args:
        content_app (str): Name of the content app.
        dirpath (str): Path to the directory containing the markdown files.
    
    Returns:
        Tuple[str, str]: content_dir_path and template_dir.
    
    Raises:
        CommandError: If the content app is not found in the directory.
    """
    try:
        base_path = "/".join(dirpath.split("/")[:-1])
        content_app_path = os.path.join(base_path, content_app)

        if not os.path.exists(content_app_path):
            raise CommandError(f"Content app {content_app} not found in {base_path}")

        content_dir_path = content_app_path
        template_dir = setup_template_directory(content_dir_path, content_app)

        return content_dir_path, template_dir
    except Exception as e:
        raise CommandError(f"Could not set up content dir path: {str(e)}")

def setup_template_directory(content_dir_path: str, content_app: str):
    """
    Set up the template directory structure.
    
    Args:
        content_dir_path (str): Path to the content app directory.
        content_app (str): Name of the content app.
    
    Returns:
        str: Path to the template directory.
    
    Raises:
        CommandError: If the template directory could not be created.
    """
    try:
        base_template_dir = Path(content_dir_path) / 'templates' / content_app / 'spellbook_md'
        base_template_dir.mkdir(parents=True, exist_ok=True)
        return str(base_template_dir)
    except Exception as e:
        raise CommandError(f"Could not create template directory: {str(e)}")

def get_folder_list(dirpath: str, md_file_path: str):
    """
    Get list of folders from dirpath relative to md_file_path.
    
    Args:
        dirpath (str): Path to the directory containing the markdown files.
        md_file_path (str): Path to the markdown file.
    
    Returns:
        List[str]: List of folders.
    """
    logger.debug(f"Getting folder list for: {dirpath}")
    folder_split = dirpath.split("/")
    folder_list = []

    # trackers
    done = False
    n = -1
    while not done:
        dirname = folder_split[n]
        logger.debug(f"Processing dirname: {dirname}")
        if dirname == str(md_file_path).split("/")[-1]:
            done = True
            break
        else:
            folder_list.append(dirname)
            n -= 1

    logger.debug(f"Generated folder list: {folder_list}")
    return folder_list


# django_spellbook/management/commands/processing/generator_utils.py

import os
import logging
import re
from pathlib import Path
from typing import Dict, Any, List, Optional

logger = logging.getLogger(__name__)

def remove_leading_dash(text: str) -> str:
    """Remove leading dash from text."""
    return text.lstrip('-')

def get_clean_url(url_pattern: str) -> str:
    """Create a clean URL from a pattern by removing leading dashes."""
    parts = url_pattern.split('/')
    clean_parts = [remove_leading_dash(part) for part in parts]
    return '/'.join(clean_parts)

def generate_view_name(url_pattern: str) -> str:
    """Generate a valid Python identifier for a view name from a URL pattern."""
    # Split by slashes first to handle each path component separately
    parts = url_pattern.split('/')
    
    # Process each part to clean dashes
    cleaned_parts = []
    for part in parts:
        # Remove leading dashes and replace internal dashes with underscores
        cleaned_part = part.lstrip('-')
        cleaned_part = cleaned_part.replace('-', '_')
        cleaned_parts.append(cleaned_part)
    
    # Join with underscores (replacing slashes with underscores)
    view_name = '_'.join(cleaned_parts)
    
    return view_name

def get_template_path(content_app: str, relative_url: str) -> str:
    """Generate template path from relative URL."""
    return os.path.join(content_app, 'spellbook_md', relative_url + '.html')

def get_spellbook_dir() -> str:
    """Get the django_spellbook base directory."""
    return os.path.abspath(os.path.join(os.path.dirname(__file__), '../../..'))

def create_file_if_not_exists(file_path: str, content: str) -> None:
    """Create a file with initial content if it doesn't exist."""
    if not os.path.exists(file_path):
        try:
            # Ensure the directory exists
            os.makedirs(os.path.dirname(file_path), exist_ok=True)
            
            with open(file_path, 'w') as f:
                f.write(content)
            logger.debug(f"Created new file: {file_path}")
        except IOError as e:
            from django.core.management.base import CommandError
            raise CommandError(f"Failed to create {file_path}: {str(e)}")
            
def write_file(file_path: str, content: str) -> None:
    """Write content to a file, ensuring directory exists."""
    try:
        # Ensure the directory exists
        os.makedirs(os.path.dirname(file_path), exist_ok=True)
        
        with open(file_path, 'w') as f:
            f.write(content)
    except IOError as e:
        from django.core.management.base import CommandError
        raise CommandError(f"Failed to write to {file_path}: {str(e)}")


# django_spellbook/management/commands/spellbook_md/processor.py

import os
import logging
from typing import List, Dict, Tuple, Optional, Any, Union
from pathlib import Path

from django.core.management.base import CommandError

from django_spellbook.management.commands.processing.file_processor import (
    MarkdownFileProcessor, ProcessedFile, MarkdownProcessingError
)
from django_spellbook.management.commands.processing.template_generator import TemplateGenerator, TemplateError
from django_spellbook.management.commands.processing.url_view_generator import URLViewGenerator, URLGenerationError
from django_spellbook.management.commands.command_utils import get_folder_list
from django_spellbook.markdown.frontmatter import FrontMatterParser
from django_spellbook.markdown.toc import TOCGenerator, TOCEntry

logger = logging.getLogger(__name__)

class MarkdownProcessorError(Exception):
    """Base exception for all markdown processor errors."""
    pass

class TOCBuildingError(MarkdownProcessorError):
    """Exception raised when building the table of contents fails."""
    pass

class FileProcessingError(MarkdownProcessorError):
    """Exception raised when processing a markdown file fails."""
    pass

class MarkdownProcessor:
    """
    Handles processing of markdown files into HTML templates and URL/view generation.
    
    This class orchestrates the entire process of converting markdown files to HTML
    templates and generating the necessary URLs and views to make the content accessible.
    
    Attributes:
        content_app (str): Name of the content app
        source_path (Path): Path to the source directory
        content_dir_path (Path): Path to the content app directory
        template_dir (Path): Path to the template directory
        file_processor (MarkdownFileProcessor): Processor for individual markdown files
        template_generator (TemplateGenerator): Generator for HTML templates
        url_generator (URLViewGenerator): Generator for URLs and views
    
    Args:
        content_app: Name of the content app
        source_path: Path to the source directory
        content_dir_path: Path to the content app directory
        template_dir: Path to the template directory
    """
    
    def __init__(
        self, 
        content_app: str, 
        source_path: Union[str, Path], 
        content_dir_path: Union[str, Path], 
        template_dir: Union[str, Path]
    ):
        self.content_app: str = content_app
        self.source_path: Path = Path(source_path)
        self.content_dir_path: Path = Path(content_dir_path)
        self.template_dir: Path = Path(template_dir)
        
        # Initialize sub-processors
        self.file_processor = MarkdownFileProcessor()
        self.file_processor.current_source_path = str(self.source_path)
        self.template_generator = TemplateGenerator(content_app, str(self.template_dir))
        self.url_generator = URLViewGenerator(
            content_app=content_app,
            content_dir_path=str(self.content_dir_path),
            source_path=str(self.source_path)
        )
        
        logger.debug(f"Initialized MarkdownProcessor for app {content_app}")
    
    def build_toc(self) -> Dict[str, TOCEntry]:
        """
        Build complete table of contents from all markdown files in the source path.
        
        This method walks through the source directory, extracts metadata from each
        markdown file, and builds a hierarchical table of contents structure.
        
        Returns:
            Dict[str, TOCEntry]: Complete table of contents mapping paths to TOC entries
            
        Raises:
            TOCBuildingError: If there is an error building the table of contents
        """
        toc_generator = TOCGenerator()
        logger.info(f"Building table of contents from {self.source_path}")

        try:
            for dirpath, dirnames, filenames in os.walk(str(self.source_path)):
                md_files = [f for f in filenames if f.endswith('.md')]
                for filename in md_files:
                    try:
                        file_path = Path(dirpath) / filename
                        relative_path = file_path.relative_to(self.source_path)
                        url = f"{self.content_app}:" + str(relative_path.with_suffix('')).replace('\\', '/')
                        url = url.replace('/', '_')
                        
                        # Get title from frontmatter
                        with open(file_path, 'r', encoding='utf-8') as f:
                            content = f.read()
                        frontmatter = FrontMatterParser(content, str(file_path))
                        title = frontmatter.metadata.get('title', relative_path.stem)

                        logger.debug(f"Adding TOC entry for {relative_path}: {title}")
                        toc_generator.add_entry(relative_path, title, url)
                    except Exception as e:
                        logger.error(f"Error adding TOC entry for {filename}: {str(e)}")
                        # Continue with next file instead of failing completely
                        continue
            
            toc = toc_generator.get_toc()
            logger.info(f"Built TOC with {len(toc)} entries")
            return toc
            
        except Exception as e:
            error_msg = f"Failed to build table of contents: {str(e)}"
            logger.error(error_msg)
            raise TOCBuildingError(error_msg) from e
    
    def process_file(
        self, 
        dirpath: str, 
        filename: str, 
        complete_toc: Dict[str, TOCEntry]
    ) -> ProcessedFile:
        """
        Process a single markdown file, converting it to HTML and creating a template.
        
        Args:
            dirpath: Path to the directory containing the markdown file
            filename: Name of the markdown file
            complete_toc: Complete table of contents
            
        Returns:
            ProcessedFile: Object containing the processed file information
            
        Raises:
            FileProcessingError: If there is an error processing the file
        """
        logger.info(f"Processing file: {filename}")
        file_path = Path(dirpath) / filename
        
        # Validate inputs
        if not file_path.exists():
            error_msg = f"File does not exist: {file_path}"
            logger.error(error_msg)
            raise FileProcessingError(error_msg)
            
        if not filename.endswith('.md'):
            error_msg = f"Not a markdown file: {filename}"
            logger.error(error_msg)
            raise FileProcessingError(error_msg)
        
        try:
            # Get folder structure for path construction
            folder_list = get_folder_list(dirpath, str(self.source_path))
            
            # Process markdown to HTML
            html_content, processed_path, context = self.file_processor.process_file(
                Path(dirpath), dirpath, filename, folder_list
            )
            
            # Generate relative paths for templates and URLs
            relative_path = Path(processed_path).relative_to(self.source_path)
            template_path = self.template_generator.get_template_path(
                filename, folder_list
            )
            relative_url = str(relative_path.with_suffix('')).replace('\\', '/')
            
            # Add TOC to context
            context.toc = complete_toc
            
            # Create the processed file object
            processed_file = ProcessedFile(
                original_path=str(processed_path),
                html_content=html_content,
                template_path=template_path,
                relative_url=relative_url,
                context=context
            )
            
            # Create the actual template file
            self.template_generator.create_template(
                template_path,
                html_content
            )
            
            logger.debug(f"Successfully processed {filename} to {template_path}")
            return processed_file
            
        except MarkdownProcessingError as e:
            error_msg = f"Error processing markdown file {filename}: {str(e)}"
            logger.error(error_msg)
            raise FileProcessingError(error_msg) from e
        except TemplateError as e:
            error_msg = f"Error creating template for {filename}: {str(e)}"
            logger.error(error_msg)
            raise FileProcessingError(error_msg) from e
        except Exception as e:
            error_msg = f"Unexpected error processing {filename}: {str(e)}"
            logger.error(error_msg)
            raise FileProcessingError(error_msg) from e
    
    def generate_urls_and_views(
        self, 
        processed_files: List[ProcessedFile], 
        complete_toc: Dict[str, TOCEntry]
    ) -> None:
        """
        Generate URLs and views for all processed files.
        
        This method creates the necessary URL patterns and view functions to make
        the processed markdown content accessible via the web.
        
        Args:
            processed_files: List of successfully processed files
            complete_toc: Complete table of contents
            
        Raises:
            MarkdownProcessorError: If there is an error generating URLs and views
        """
        logger.info(f"Generating URLs and views for {len(processed_files)} files")
        
        if not processed_files:
            logger.warning("No processed files to generate URLs and views for")
            return
            
        try:
            self.url_generator.generate_urls_and_views(processed_files, complete_toc)
            logger.info(f"Successfully generated URLs and views for {len(processed_files)} files")
        except URLGenerationError as e:
            error_msg = f"Error generating URLs and views: {str(e)}"
            logger.error(error_msg)
            raise MarkdownProcessorError(error_msg) from e
        except Exception as e:
            error_msg = f"Unexpected error generating URLs and views: {str(e)}"
            logger.error(error_msg)
            raise MarkdownProcessorError(error_msg) from e


import os
import logging
from typing import List, Dict, Tuple
from pathlib import Path

from django.core.management.base import BaseCommand, CommandError
from django.conf import settings

from django_spellbook.management.commands.command_utils import (
    validate_spellbook_settings,
    setup_directory_structure
)
from django_spellbook.management.commands.spellbook_md_p.discovery import (
    discover_blocks,
    find_markdown_files
)
from django_spellbook.management.commands.spellbook_md_p.processor import (
    MarkdownProcessor
)

logger = logging.getLogger(__name__)

# Custom Exception Classes
class SpellbookMDError(CommandError):
    """Base exception for all Spellbook MD errors."""
    pass

class ConfigurationError(SpellbookMDError):
    """Error related to configuration issues (settings, paths, etc.)."""
    pass

class ContentDiscoveryError(SpellbookMDError):
    """Error related to finding and parsing content files."""
    pass

class ProcessingError(SpellbookMDError):
    """Error during markdown processing."""
    pass

class OutputGenerationError(SpellbookMDError):
    """Error generating templates, views, or URLs."""
    pass

class Command(BaseCommand):
    help = "Converts markdown to html, with a spellbook twist"

    def add_arguments(self, parser):
        parser.add_argument(
            '--continue-on-error',
            action='store_true',
            help='Continue processing files even if some fail'
        )

    def handle(self, *args, **options):
        """Process markdown files from all configured source-destination pairs."""
        self.continue_on_error = options.get('continue_on_error', False)
        
        try:
            # Discover spell blocks
            try:
                block_count = discover_blocks(self.stdout)
                self.stdout.write(f"Discovered {block_count} SpellBlocks")
            except Exception as e:
                self.stdout.write(
                    self.style.WARNING(
                        f"Error during block discovery: {str(e)}. "
                        "Processing will continue but some content may not render correctly."
                    )
                )
                logger.warning(f"Block discovery error: {str(e)}", exc_info=True)
            
            # Validate settings
            try:
                md_file_paths, content_apps = validate_spellbook_settings()
            except Exception as e:
                error_message = f"Configuration error: {str(e)}"
                self.stdout.write(self.style.ERROR(error_message))
                self.stdout.write(
                    "Please check your Django settings.py file and ensure "
                    "SPELLBOOK_MD_PATH and SPELLBOOK_MD_APP are correctly configured."
                )
                logger.error(f"Settings validation error: {str(e)}", exc_info=True)
                raise ConfigurationError(f"Settings validation failed: {str(e)}")
            
            # Process each source-destination pair
            pair_results = []
            for i, (md_path, content_app) in enumerate(zip(md_file_paths, content_apps)):
                pair_name = f"pair {i+1}/{len(md_file_paths)}: {md_path} → {content_app}"
                self.stdout.write(
                    self.style.SUCCESS(f"Processing source-destination {pair_name}")
                )
                
                try:
                    processed_count = self._process_source_destination_pair(md_path, content_app)
                    pair_results.append((md_path, content_app, True, processed_count))
                except Exception as e:
                    error_message = f"Error processing {pair_name}: {str(e)}"
                    self.stdout.write(self.style.ERROR(error_message))
                    # Always use 0 as the count for failed pairs to avoid None
                    pair_results.append((md_path, content_app, False, 0))
                    logger.error(f"Error processing {pair_name}: {str(e)}", exc_info=True)
                    
                    if len(md_file_paths) > 1:
                        self.stdout.write("Continuing with next pair...")
                        continue
                    else:
                        raise
            
            # Summary report
            self.stdout.write("\nProcessing Summary:")
            success_count = sum(1 for _, _, success, _ in pair_results if success)
            total_processed = sum(count for _, _, success, count in pair_results if success)
            
            if success_count == len(pair_results):
                self.stdout.write(self.style.SUCCESS(
                    f"All {len(pair_results)} source-destination pairs processed successfully. "
                    f"Total files processed: {total_processed}."
                ))
            else:
                failed_pairs = [(src, dst) for src, dst, success, _ in pair_results if not success]
                self.stdout.write(self.style.WARNING(
                    f"{success_count} of {len(pair_results)} pairs processed successfully. "
                    f"Total files processed: {total_processed}. "
                    f"Failed pairs: {', '.join(f'{src} → {dst}' for src, dst in failed_pairs)}"
                ))
                
        except Exception as e:
            error_message = f"Command failed: {str(e)}"
            self.stdout.write(self.style.ERROR(error_message))
            logger.error(f"Command failed: {str(e)}", exc_info=True)
            raise

    def _process_source_destination_pair(self, md_path, content_app):
        """Process all markdown files for a single source-destination pair."""
        # Find markdown files
        try:
            markdown_files = find_markdown_files(md_path)
            
            if not markdown_files:
                error_message = f"No markdown files found in {md_path}"
                self.stdout.write(self.style.ERROR(error_message))
                self.stdout.write(
                    f"Make sure the directory exists and contains .md files."
                )
                raise ContentDiscoveryError(error_message)
                
            self.stdout.write(f"Found {len(markdown_files)} markdown files to process")
            
        except ContentDiscoveryError:
            raise
        except Exception as e:
            logger.error(f"Error discovering markdown files: {str(e)}", exc_info=True)
            raise ContentDiscoveryError(f"Failed to discover markdown files: {str(e)}")
        
        # Set up directory structure
        try:
            first_dirpath = markdown_files[0][0]
            content_dir_path, template_dir = setup_directory_structure(content_app, first_dirpath)
        except Exception as e:
            error_message = f"Failed to set up directory structure: {str(e)}"
            self.stdout.write(self.style.ERROR(f"Directory setup error: {str(e)}"))
            self.stdout.write(
                "Check that the content app exists and is correctly configured in your Django project."
            )
            logger.error(f"Directory setup error: {str(e)}", exc_info=True)
            raise ConfigurationError(error_message)
        
        # Initialize processor
        try:
            processor = MarkdownProcessor(
                content_app=content_app,
                source_path=md_path,
                content_dir_path=content_dir_path,
                template_dir=template_dir
            )
        except Exception as e:
            logger.error(f"Error initializing markdown processor: {str(e)}", exc_info=True)
            raise ProcessingError(f"Failed to initialize markdown processor: {str(e)}")
        
        # Build TOC
        try:
            self.stdout.write("Building table of contents...")
            complete_toc = processor.build_toc()
        except Exception as e:
            self.stdout.write(
                self.style.WARNING(
                    f"Error building table of contents: {str(e)}. "
                    "Processing will continue but navigation links may not work correctly."
                )
            )
            logger.warning(f"TOC building error: {str(e)}", exc_info=True)
            complete_toc = {}
        
        # Process each file
        processed_files = []
        error_files = []
        
        for i, (dirpath, filename) in enumerate(markdown_files):
            file_path = os.path.join(dirpath, filename)
            self.stdout.write(f"Processing file {i+1}/{len(markdown_files)}: {filename}")
            
            try:
                processed_file = processor.process_file(dirpath, filename, complete_toc)
                
                if processed_file:
                    processed_files.append(processed_file)
                else:
                    error_files.append(file_path)
                    self.stdout.write(
                        self.style.WARNING(f"File not processed: {filename}")
                    )
                    
            except Exception as e:
                error_files.append(file_path)
                error_message = f"Error processing file {filename}: {str(e)}"
                self.stdout.write(self.style.ERROR(error_message))
                logger.error(f"File processing error: {str(e)}", exc_info=True)
                
                if not self.continue_on_error:
                    raise ProcessingError(f"Failed to process file {filename}: {str(e)}")
        
        # Generate URLs and views
        if processed_files:
            try:
                self.stdout.write("Generating URLs and views...")
                processor.generate_urls_and_views(processed_files, complete_toc)
            except Exception as e:
                error_message = f"Error generating URLs and views: {str(e)}"
                self.stdout.write(self.style.ERROR(error_message))
                logger.error(f"URL and view generation error: {str(e)}", exc_info=True)
                raise OutputGenerationError(f"Failed to generate URLs and views: {str(e)}")
            
            self.stdout.write(
                self.style.SUCCESS(
                    f"Successfully processed {len(processed_files)} files for {content_app}"
                )
            )
            
            if error_files:
                self.stdout.write(
                    self.style.WARNING(
                        f"Note: {len(error_files)} files could not be processed: "
                        f"{', '.join(os.path.basename(f) for f in error_files[:5])}"
                        f"{' and more...' if len(error_files) > 5 else ''}"
                    )
                )
        else:
            error_message = f"No markdown files were processed successfully for {content_app}"
            self.stdout.write(self.style.ERROR(error_message))
            self.stdout.write(
                "Check the markdown syntax and structure of your files, and ensure all required SpellBlocks are available."
            )
            raise ProcessingError(error_message)
        
        return len(processed_files)


# django_spellbook/tests/test_processor.py
import unittest
from unittest.mock import patch, Mock, MagicMock, mock_open
from io import StringIO
from pathlib import Path
import os

from django.test import TestCase

from django_spellbook.management.commands.spellbook_md_p.processor import (
    MarkdownProcessor,
    MarkdownProcessorError,
    TOCBuildingError,
    FileProcessingError
)
from django_spellbook.management.commands.processing.file_processor import (
    ProcessedFile, MarkdownProcessingError
)
from django_spellbook.management.commands.processing.template_generator import TemplateError
from django_spellbook.management.commands.processing.url_view_generator import URLGenerationError
from django_spellbook.markdown.toc import TOCEntry

class TestMarkdownProcessor(TestCase):
    """Test normal functionality of the MarkdownProcessor class."""
    
    def setUp(self):
        """Set up common test variables."""
        self.content_app = 'test_app'
        self.source_path = '/test/source'
        self.content_dir_path = '/test/app'
        self.template_dir = '/test/templates'
        
        # Create MarkdownProcessor with mocked dependencies
        with patch('django_spellbook.management.commands.spellbook_md_p.processor.MarkdownFileProcessor'), \
             patch('django_spellbook.management.commands.spellbook_md_p.processor.TemplateGenerator'), \
             patch('django_spellbook.management.commands.spellbook_md_p.processor.URLViewGenerator'):
            self.processor = MarkdownProcessor(
                self.content_app,
                self.source_path,
                self.content_dir_path,
                self.template_dir
            )
            
        # Set up mocks for the sub-processors
        self.processor.file_processor = Mock()
        self.processor.template_generator = Mock()
        self.processor.url_generator = Mock()
    
    def test_initialization(self):
        """Test initialization with different types of paths."""
        # Test with string paths
        with patch('django_spellbook.management.commands.spellbook_md_p.processor.MarkdownFileProcessor'), \
             patch('django_spellbook.management.commands.spellbook_md_p.processor.TemplateGenerator'), \
             patch('django_spellbook.management.commands.spellbook_md_p.processor.URLViewGenerator'):
            processor = MarkdownProcessor(
                'test_app',
                '/test/source',
                '/test/app',
                '/test/templates'
            )
            self.assertEqual(processor.content_app, 'test_app')
            self.assertEqual(processor.source_path, Path('/test/source'))
            self.assertEqual(processor.content_dir_path, Path('/test/app'))
            self.assertEqual(processor.template_dir, Path('/test/templates'))
        
        # Test with Path objects
        with patch('django_spellbook.management.commands.spellbook_md_p.processor.MarkdownFileProcessor'), \
             patch('django_spellbook.management.commands.spellbook_md_p.processor.TemplateGenerator'), \
             patch('django_spellbook.management.commands.spellbook_md_p.processor.URLViewGenerator'):
            processor = MarkdownProcessor(
                'test_app',
                Path('/test/source'),
                Path('/test/app'),
                Path('/test/templates')
            )
            self.assertEqual(processor.content_app, 'test_app')
            self.assertEqual(processor.source_path, Path('/test/source'))
            self.assertEqual(processor.content_dir_path, Path('/test/app'))
            self.assertEqual(processor.template_dir, Path('/test/templates'))
    
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.os.walk')
    @patch('builtins.open', new_callable=mock_open, read_data="---\ntitle: Test Title\n---\nContent")
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.Path.relative_to')
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.FrontMatterParser')
    def test_build_toc(self, mock_frontmatter, mock_relative_to, mock_file, mock_walk):
        """Test building the table of contents."""
        # Setup mocks
        mock_walk.return_value = [
            ('/test/source', ['dir1'], ['file1.md', 'file2.txt']),
            ('/test/source/dir1', [], ['---file3.md'])
        ]
        
        # Setup relative_to to return predictable values
        def relative_to_side_effect(base_path):
            path_str = str(mock_relative_to.call_args[0][0])
            if 'file1.md' in path_str:
                return Path('file1.md')
            else:
                return Path('dir1/file3.md')
        
        mock_relative_to.side_effect = relative_to_side_effect
        
        # Setup frontmatter to return metadata with titles
        mock_frontmatter_instance = Mock()
        mock_frontmatter_instance.metadata = {'title': 'Test Title'}
        mock_frontmatter.return_value = mock_frontmatter_instance
        
        # Call the method
        toc = self.processor.build_toc()
        
        # The TOC should be a dict with keys 'title', 'url', and 'children'
        self.assertIn('title', toc)
        self.assertIn('url', toc)
        self.assertIn('children', toc)
        
        # Verify children structure
        children = toc['children']
        
        # Check if dir1 is in the structure
        self.assertIn('dir1', children)
        
        # Verify file3 is in dir1's children
        dir1_children = children['dir1']['children']
        self.assertIn('file3', dir1_children)  # The stem of file3.md
        
        # Verify title for file3
        self.assertEqual(dir1_children['file3']['title'], 'Test Title')
        
        # Veriify url for file3
        self.assertEqual(dir1_children['file3']['url'], 'test_app:dir1_file3')
        
        # Verify the open calls (should be called for each markdown file)
        self.assertEqual(mock_file.call_count, 2)
        
        # Verify frontmatter parser was called for each file
        self.assertEqual(mock_frontmatter.call_count, 2)
    
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.Path.exists')
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.Path.relative_to')
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.get_folder_list')
    def test_process_file(self, mock_get_folder_list, mock_relative_to, mock_exists):
        """Test processing a single markdown file."""
        # Setup mocks
        mock_exists.return_value = True
        mock_get_folder_list.return_value = ['folder1', 'folder2']
        mock_relative_to.return_value = Path('folder1/folder2/test.md')
        
        # Setup file processor mock
        self.processor.file_processor.process_file.return_value = (
            '<h1>Test</h1>',  # HTML content
            '/test/source/folder1/folder2/test.md',  # Processed path
            MagicMock()  # Context
        )
        
        # Setup template generator mock
        self.processor.template_generator.get_template_path.return_value = 'template/path.html'
        
        # Call the method
        toc = {
            'folder1/folder2/test.md': TOCEntry(Path('folder1/folder2/test.md'), 'Test Title', 'test_app:folder1_folder2_test')
        }
        result = self.processor.process_file('/test/source/folder1/folder2', 'test.md', toc)
        
        # Verify results
        self.assertIsInstance(result, ProcessedFile)
        self.assertEqual(result.original_path, '/test/source/folder1/folder2/test.md')
        self.assertEqual(result.html_content, '<h1>Test</h1>')
        self.assertEqual(result.template_path, 'template/path.html')
        self.assertEqual(result.relative_url, 'folder1/folder2/test')
        
        # Verify method calls
        mock_get_folder_list.assert_called_once_with('/test/source/folder1/folder2', str(self.processor.source_path))
        self.processor.file_processor.process_file.assert_called_once()
        self.processor.template_generator.get_template_path.assert_called_once_with('test.md', ['folder1', 'folder2'])
        self.processor.template_generator.create_template.assert_called_once_with('template/path.html', '<h1>Test</h1>')
    
    def test_generate_urls_and_views(self):
        """Test generating URLs and views."""
        # Setup mocks
        processed_files = [MagicMock(spec=ProcessedFile) for _ in range(3)]
        toc = {'path1': MagicMock(spec=TOCEntry), 'path2': MagicMock(spec=TOCEntry)}
        
        # Call the method
        self.processor.generate_urls_and_views(processed_files, toc)
        
        # Verify method calls
        self.processor.url_generator.generate_urls_and_views.assert_called_once_with(processed_files, toc)
    
    def test_generate_urls_and_views_empty(self):
        """Test generating URLs and views with empty processed files."""
        # Call the method with empty list
        self.processor.generate_urls_and_views([], {})
        
        # Verify no method calls
        self.processor.url_generator.generate_urls_and_views.assert_not_called()


class TestMarkdownProcessorExceptions(TestCase):
    """Test exception handling in the MarkdownProcessor class."""
    
    def setUp(self):
        """Set up common test variables."""
        self.content_app = 'test_app'
        self.source_path = '/test/source'
        self.content_dir_path = '/test/app'
        self.template_dir = '/test/templates'
        
        # Create MarkdownProcessor with mocked dependencies
        with patch('django_spellbook.management.commands.spellbook_md_p.processor.MarkdownFileProcessor'), \
             patch('django_spellbook.management.commands.spellbook_md_p.processor.TemplateGenerator'), \
             patch('django_spellbook.management.commands.spellbook_md_p.processor.URLViewGenerator'):
            self.processor = MarkdownProcessor(
                self.content_app,
                self.source_path,
                self.content_dir_path,
                self.template_dir
            )
            
        # Set up mocks for the sub-processors
        self.processor.file_processor = Mock()
        self.processor.template_generator = Mock()
        self.processor.url_generator = Mock()
    
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.os.walk')
    def test_build_toc_os_error(self, mock_walk):
        """Test TOC building error when os.walk fails."""
        # Setup mock to raise exception
        mock_walk.side_effect = OSError("Permission denied")
        
        # Verify exception is caught and re-raised
        with self.assertRaises(TOCBuildingError):
            self.processor.build_toc()
    
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.os.walk')
    @patch('builtins.open')
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.TOCGenerator')
    def test_build_toc_file_error(self, mock_toc_generator_class, mock_open, mock_walk):
        """Test TOC building error when file cannot be read."""
        # Setup mocks
        mock_walk.return_value = [('/test/source', [], ['file1.md'])]
        mock_open.side_effect = IOError("Cannot read file")
        
        # Mock TOCGenerator and its instance
        mock_toc_generator = Mock()
        mock_toc_generator_class.return_value = mock_toc_generator
        
        # Setup the mock TOC that will be returned - empty because file read failed
        mock_toc = {}
        mock_toc_generator.get_toc.return_value = mock_toc
        
        # Verify exception in file read is caught and function continues
        toc = self.processor.build_toc()
        self.assertEqual(len(toc), 0)  # No entries due to file error
        
        # Verify get_toc was called despite file error
        mock_toc_generator.get_toc.assert_called_once()
        # Verify add_entry was not called (couldn't read the file)
        mock_toc_generator.add_entry.assert_not_called()
        
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.Path.exists')
    def test_process_file_not_exists(self, mock_exists):
        """Test processing file that doesn't exist."""
        # Setup mock
        mock_exists.return_value = False
        
        # Call method and verify exception
        with self.assertRaises(FileProcessingError):
            self.processor.process_file('/test/source', 'nonexistent.md', {})
    
    def test_process_file_not_markdown(self):
        """Test processing file that is not a markdown file."""
        # Call method with non-markdown file
        with patch('django_spellbook.management.commands.spellbook_md_p.processor.Path.exists', return_value=True):
            with self.assertRaises(FileProcessingError):
                self.processor.process_file('/test/source', 'file.txt', {})
    
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.Path.exists')
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.get_folder_list')
    def test_process_file_processing_error(self, mock_get_folder_list, mock_exists):
        """Test error during markdown processing."""
        # Setup mocks
        mock_exists.return_value = True
        mock_get_folder_list.return_value = ['folder']
        
        # Setup file processor to raise exception
        self.processor.file_processor.process_file.side_effect = MarkdownProcessingError("Processing error")
        
        # Call method and verify exception
        with self.assertRaises(FileProcessingError):
            self.processor.process_file('/test/source', 'file.md', {})
    
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.Path.exists')
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.get_folder_list')
    def test_process_file_template_error(self, mock_get_folder_list, mock_exists):
        """Test error during template creation."""
        # Setup mocks
        mock_exists.return_value = True
        mock_get_folder_list.return_value = ['folder']
        
        # Setup file processor and template generator
        self.processor.file_processor.process_file.return_value = (
            '<h1>Test</h1>',  # HTML content
            '/test/source/file.md',  # Processed path
            MagicMock()  # Context
        )
        self.processor.template_generator.get_template_path.return_value = 'template/path.html'
        self.processor.template_generator.create_template.side_effect = TemplateError("Template error")
        
        # Call method and verify exception
        with self.assertRaises(FileProcessingError):
            self.processor.process_file('/test/source', 'file.md', {})
    
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.Path.exists')
    @patch('django_spellbook.management.commands.spellbook_md_p.processor.get_folder_list')
    def test_process_file_unexpected_error(self, mock_get_folder_list, mock_exists):
        """Test unexpected error during file processing."""
        # Setup mocks
        mock_exists.return_value = True
        mock_get_folder_list.return_value = ['folder']
        
        # Setup file processor to raise unexpected exception
        self.processor.file_processor.process_file.side_effect = ValueError("Unexpected error")
        
        # Call method and verify exception
        with self.assertRaises(FileProcessingError):
            self.processor.process_file('/test/source', 'file.md', {})
    
    def test_generate_urls_and_views_error(self):
        """Test error during URL and view generation."""
        # Setup processed files and TOC
        processed_files = [MagicMock(spec=ProcessedFile) for _ in range(2)]
        toc = {'path': MagicMock(spec=TOCEntry)}
        
        # Setup URL generator to raise exception
        self.processor.url_generator.generate_urls_and_views.side_effect = URLGenerationError("URL generation error")
        
        # Call method and verify exception
        with self.assertRaises(MarkdownProcessorError):
            self.processor.generate_urls_and_views(processed_files, toc)
    
    def test_generate_urls_and_views_unexpected_error(self):
        """Test unexpected error during URL and view generation."""
        # Setup processed files and TOC
        processed_files = [MagicMock(spec=ProcessedFile) for _ in range(2)]
        toc = {'path': MagicMock(spec=TOCEntry)}
        
        # Setup URL generator to raise unexpected exception
        self.processor.url_generator.generate_urls_and_views.side_effect = ValueError("Unexpected error")
        
        # Call method and verify exception
        with self.assertRaises(MarkdownProcessorError):
            self.processor.generate_urls_and_views(processed_files, toc)


# django_spellbook/tests/test_command_utils.py

import os
import unittest
from pathlib import Path
from unittest.mock import patch, Mock

from django.test import TestCase, override_settings
from django.core.management.base import CommandError

from django_spellbook.management.commands.command_utils import (
    normalize_settings,
    validate_spellbook_settings,
    _validate_setting_values,
    setup_directory_structure,
    setup_template_directory,
    get_folder_list
)

class TestNormalizeSettings(TestCase):
    """Tests for normalize_settings function."""

    def test_normalize_string_values(self):
        """Test normalizing string values."""
        md_paths, md_apps = normalize_settings('/test/path', 'test_app')
        
        self.assertEqual(md_paths, ['/test/path'])
        self.assertEqual(md_apps, ['test_app'])
    
    def test_normalize_path_object(self):
        """Test normalizing Path object."""
        path_obj = Path('/test/path')
        md_paths, md_apps = normalize_settings(path_obj, 'test_app')
        
        self.assertEqual(md_paths, [path_obj])
        self.assertEqual(md_apps, ['test_app'])
    
    def test_normalize_list_values(self):
        """Test normalizing list values."""
        md_paths, md_apps = normalize_settings(
            ['/test/path1', '/test/path2'],
            ['app1', 'app2']
        )
        
        self.assertEqual(md_paths, ['/test/path1', '/test/path2'])
        self.assertEqual(md_apps, ['app1', 'app2'])
    
    def test_normalize_mixed_values(self):
        """Test normalizing mixed values (string and list)."""
        md_paths, md_apps = normalize_settings('/test/path', ['app1', 'app2'])
        
        self.assertEqual(md_paths, ['/test/path'])
        self.assertEqual(md_apps, ['app1', 'app2'])


class TestValidateSpellbookSettings(TestCase):
    """Tests for validate_spellbook_settings function."""

    @override_settings(SPELLBOOK_MD_PATH='/test/path', SPELLBOOK_MD_APP='test_app')
    def test_settings_with_new_names(self):
        """Test validation with new setting names."""
        md_paths, md_apps = validate_spellbook_settings()
        
        self.assertEqual(md_paths, ['/test/path'])
        self.assertEqual(md_apps, ['test_app'])
    
    @override_settings(
        SPELLBOOK_MD_PATH='/test/path',
        SPELLBOOK_CONTENT_APP='test_app',
        SPELLBOOK_MD_APP=None
    )
    @patch('django_spellbook.management.commands.command_utils.logger')
    def test_settings_with_old_names(self, mock_logger):
        """Test validation with old setting names (deprecated)."""
        md_paths, md_apps = validate_spellbook_settings()
        
        self.assertEqual(md_paths, ['/test/path'])
        self.assertEqual(md_apps, ['test_app'])
        
        # Check that we logged a deprecation warning
        mock_logger.warning.assert_called_with(
            "SPELLBOOK_CONTENT_APP is deprecated, use SPELLBOOK_MD_APP instead."
        )
    
    @override_settings(
        SPELLBOOK_MD_PATH=['/path1', '/path2'],
        SPELLBOOK_MD_APP=['app1', 'app2']
    )
    def test_settings_with_multiple_pairs(self):
        """Test validation with multiple source-destination pairs."""
        md_paths, md_apps = validate_spellbook_settings()
        
        self.assertEqual(md_paths, ['/path1', '/path2'])
        self.assertEqual(md_apps, ['app1', 'app2'])
    
    @override_settings(
        SPELLBOOK_MD_PATH=None,
        SPELLBOOK_MD_APP=None,
        SPELLBOOK_CONTENT_APP=None
    )
    def test_settings_missing(self):
        """Test validation with missing settings."""
        with self.assertRaises(CommandError) as context:
            validate_spellbook_settings()
        
        self.assertIn("Missing required settings", str(context.exception))


class TestValidateSettingValues(TestCase):
    """Tests for _validate_setting_values function."""

    def test_valid_settings(self):
        """Test validation with valid settings."""
        # This should not raise any exceptions
        _validate_setting_values(['/test/path'], ['test_app'])
    
    def test_missing_path(self):
        """Test validation with missing path."""
        with self.assertRaises(CommandError) as context:
            _validate_setting_values([], ['test_app'])
        
        self.assertIn("SPELLBOOK_MD_PATH", str(context.exception))
    
    def test_missing_app(self):
        """Test validation with missing app."""
        with self.assertRaises(CommandError) as context:
            _validate_setting_values(['/test/path'], [])
        
        self.assertIn("SPELLBOOK_MD_APP or SPELLBOOK_CONTENT_APP", str(context.exception))
    
    def test_unequal_lengths(self):
        """Test validation with unequal lengths."""
        with self.assertRaises(CommandError) as context:
            _validate_setting_values(['/path1', '/path2'], ['app1'])
        
        self.assertIn("must have the same number of entries", str(context.exception))
    
    def test_empty_path(self):
        """Test validation with empty path."""
        with self.assertRaises(CommandError) as context:
            _validate_setting_values(['', '/path2'], ['app1', 'app2'])
        
        self.assertIn("Invalid SPELLBOOK_MD_PATH configuration", str(context.exception))
    
    def test_empty_app(self):
        """Test validation with empty app."""
        with self.assertRaises(CommandError) as context:
            _validate_setting_values(['/path1', '/path2'], ['app1', ''])
        
        self.assertIn("SPELLBOOK_MD_APP must be a non-empty string", str(context.exception))


class TestSetupDirectoryStructure(TestCase):
    """Tests for setup_directory_structure function."""

    @patch('os.path.exists')
    @patch('django_spellbook.management.commands.command_utils.setup_template_directory')
    def test_successful_setup(self, mock_setup_template, mock_exists):
        """Test successful directory structure setup."""
        # Setup mocks
        mock_exists.return_value = True
        mock_setup_template.return_value = '/test/app/templates/test_app/spellbook_md'
        
        # Call function
        content_dir_path, template_dir = setup_directory_structure('test_app', '/test/content/file.md')
        
        # Verify results
        self.assertEqual(content_dir_path, '/test/content/test_app')
        self.assertEqual(template_dir, '/test/app/templates/test_app/spellbook_md')
        
        # Verify correct function calls
        mock_exists.assert_called_once_with('/test/content/test_app')
        mock_setup_template.assert_called_once_with('/test/content/test_app', 'test_app')
    
    @patch('os.path.exists')
    def test_content_app_not_found(self, mock_exists):
        """Test error when content app is not found."""
        # Setup mocks
        mock_exists.return_value = False
        
        # Call function
        with self.assertRaises(CommandError) as context:
            setup_directory_structure('test_app', '/test/content/file.md')
        
        # Verify error message
        self.assertIn("Content app test_app not found", str(context.exception))
    
    @patch('os.path.exists')
    @patch('django_spellbook.management.commands.command_utils.setup_template_directory')
    def test_setup_template_error(self, mock_setup_template, mock_exists):
        """Test error when template directory setup fails."""
        # Setup mocks
        mock_exists.return_value = True
        mock_setup_template.side_effect = Exception("Template directory error")
        
        # Call function
        with self.assertRaises(CommandError) as context:
            setup_directory_structure('test_app', '/test/content/file.md')
        
        # Verify error message
        self.assertIn("Could not set up content dir path", str(context.exception))
        self.assertIn("Template directory error", str(context.exception))


class TestSetupTemplateDirectory(TestCase):
    """Tests for setup_template_directory function."""

    @patch('pathlib.Path.mkdir')
    def test_successful_setup(self, mock_mkdir):
        """Test successful template directory setup."""
        # Call function
        template_dir = setup_template_directory('/test/app', 'test_app')
        
        # Verify result
        self.assertEqual(template_dir, '/test/app/templates/test_app/spellbook_md')
        
        # Verify directory was created
        mock_mkdir.assert_called_once_with(parents=True, exist_ok=True)
    
    @patch('pathlib.Path.mkdir')
    def test_directory_creation_error(self, mock_mkdir):
        """Test error when directory creation fails."""
        # Setup mock
        mock_mkdir.side_effect = PermissionError("Permission denied")
        
        # Call function
        with self.assertRaises(CommandError) as context:
            setup_template_directory('/test/app', 'test_app')
        
        # Verify error message
        self.assertIn("Could not create template directory", str(context.exception))
        self.assertIn("Permission denied", str(context.exception))


class TestGetFolderList(unittest.TestCase):
    """Tests for get_folder_list function."""

    @patch('django_spellbook.management.commands.command_utils.logger')
    def test_get_folder_list(self, mock_logger):
        """Test getting folder list from path."""
        # Call function
        folders = get_folder_list('/test/content/subfolder/file.md', 'content')
        
        # Verify result
        self.assertEqual(folders, ['file.md', 'subfolder'])
    
    @patch('django_spellbook.management.commands.command_utils.logger')
    def test_get_folder_list_nested(self, mock_logger):
        """Test getting folder list from deeply nested path."""
        # Call function
        folders = get_folder_list('/a/b/c/d/e/f/g.md', 'b')
        
        # Verify result
        self.assertEqual(folders, ['g.md', 'f', 'e', 'd', 'c'])
    
    @patch('django_spellbook.management.commands.command_utils.logger')
    def test_folder_at_root_level(self, mock_logger):
        """Test getting folder list when at root level."""
        # Call function
        folders = get_folder_list('/test/content/file.md', 'content')
        
        # Verify result
        self.assertEqual(folders, ['file.md'])


# django_spellbook/tests/test_spellbook_md_integration.py

import unittest
from unittest.mock import patch, Mock, MagicMock
from io import StringIO

from django.test import TestCase
from django.core.management import call_command

from django_spellbook.management.commands.spellbook_md import (
    Command, ConfigurationError, ContentDiscoveryError, ProcessingError
)

class TestSpellbookMDIntegration(TestCase):
    """Integration tests for the spellbook_md command."""
    
    def test_basic_functionality(self):
        """Test basic command functionality with a single source-destination pair."""
        # Use a context manager approach for more precise control over patching
        with patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings') as mock_validate, \
             patch('django_spellbook.management.commands.spellbook_md.find_markdown_files') as mock_find_files, \
             patch('django_spellbook.management.commands.spellbook_md.setup_directory_structure') as mock_setup, \
             patch('django_spellbook.management.commands.spellbook_md.MarkdownProcessor') as mock_processor_class:
            
            # Setup mocks
            mock_validate.return_value = (['/test/path'], ['test_app'])
            mock_find_files.return_value = [('/test/path', 'test.md')]
            mock_setup.return_value = ('/app/path', '/app/templates')
            
            # Setup processor mock
            mock_processor = MagicMock()
            mock_processor.build_toc.return_value = {}
            mock_processor.process_file.return_value = MagicMock()  # Mock a successful processed file
            mock_processor_class.return_value = mock_processor
            
            # Run the command
            out = StringIO()
            call_command('spellbook_md', stdout=out)
            
            # Check output
            output = out.getvalue()
            self.assertIn('Found 1 markdown files to process', output)
            self.assertIn('Successfully processed 1 files for test_app', output)
            
            # Verify correct method calls
            mock_validate.assert_called_once()
            mock_find_files.assert_called_once_with('/test/path')
            mock_setup.assert_called_once_with('test_app', '/test/path')
            mock_processor.build_toc.assert_called_once()
            mock_processor.process_file.assert_called_once_with('/test/path', 'test.md', {})
            mock_processor.generate_urls_and_views.assert_called_once()
    
    def test_multiple_sources(self):
        """Test processing multiple source-destination pairs."""
        with patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings') as mock_validate, \
            patch('django_spellbook.management.commands.spellbook_md.find_markdown_files') as mock_find_files, \
            patch('django_spellbook.management.commands.spellbook_md.setup_directory_structure') as mock_setup, \
            patch('django_spellbook.management.commands.spellbook_md.MarkdownProcessor') as mock_processor_class:
            
            # Setup mocks
            mock_validate.return_value = (['/path1', '/path2'], ['app1', 'app2'])
            
            # Use call count tracking to ensure correct behavior for each call
            find_files_calls = 0
            def find_files_side_effect(path):
                nonlocal find_files_calls
                result = []
                if find_files_calls == 0:
                    result = [('/path1', 'file1.md'), ('/path1', 'file2.md')]
                else:
                    result = [('/path2', 'file3.md')]
                find_files_calls += 1
                return result
            
            mock_find_files.side_effect = find_files_side_effect
            
            # Track setup calls similarly
            setup_calls = 0
            def setup_side_effect(app, path):
                nonlocal setup_calls
                result = None
                if setup_calls == 0:
                    result = ('/app1/path', '/app1/templates')
                else:
                    result = ('/app2/path', '/app2/templates')
                setup_calls += 1
                return result
            
            mock_setup.side_effect = setup_side_effect
            
            # Track processor creation
            processor_calls = 0
            def processor_factory(**kwargs):
                nonlocal processor_calls
                processor = MagicMock()
                processor.build_toc.return_value = {}
                
                if processor_calls == 0:
                    # First processor processes 2 files
                    processor.process_file.side_effect = [MagicMock(), MagicMock()]
                else:
                    # Second processor processes 1 file
                    processor.process_file.return_value = MagicMock()
                
                processor_calls += 1
                return processor
            
            mock_processor_class.side_effect = processor_factory
            
            # Run the command
            out = StringIO()
            call_command('spellbook_md', stdout=out)
            
            # Check output
            output = out.getvalue()
            self.assertIn('Found 2 markdown files to process', output)  # First source
            self.assertIn('Found 1 markdown files to process', output)  # Second source
            self.assertIn('Successfully processed 2 files for app1', output)
            self.assertIn('Successfully processed 1 files for app2', output)
    
    def test_continue_on_error(self):
        """Test --continue-on-error flag allows processing to continue after file errors."""
        with patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings') as mock_validate, \
            patch('django_spellbook.management.commands.spellbook_md.find_markdown_files') as mock_find_files, \
            patch('django_spellbook.management.commands.spellbook_md.setup_directory_structure') as mock_setup, \
            patch('django_spellbook.management.commands.spellbook_md.MarkdownProcessor') as mock_processor_class:
            
            # Setup mocks
            mock_validate.return_value = (['/test/path'], ['test_app'])
            mock_find_files.return_value = [
                ('/test/path', 'good.md'),
                ('/test/path', 'bad.md')
            ]
            mock_setup.return_value = ('/app/path', '/app/templates')
            
            # Setup processor mock with a file that succeeds and one that fails
            mock_processor = MagicMock()
            mock_processor.build_toc.return_value = {}
            
            # Configure process_file to succeed for good.md but fail for bad.md
            def process_file_side_effect(dirpath, filename, toc):
                if filename == 'bad.md':
                    raise ProcessingError("Error processing file")
                return MagicMock()  # Return a mock for successful file
            
            mock_processor.process_file.side_effect = process_file_side_effect
            mock_processor_class.return_value = mock_processor
            
            # Run the command with continue-on-error flag
            out = StringIO()
            call_command('spellbook_md', continue_on_error=True, stdout=out)
            
            # Check output
            output = out.getvalue()
            self.assertIn('Error processing file bad.md', output)
            self.assertIn('Successfully processed 1 files for test_app', output)
            
            # Verify both files were attempted
            self.assertEqual(mock_processor.process_file.call_count, 2)
            mock_processor.generate_urls_and_views.assert_called_once()
        
    def test_error_in_second_source(self):
        """Test handling errors in second source while continuing processing."""
        with patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings') as mock_validate, \
            patch('django_spellbook.management.commands.spellbook_md.find_markdown_files') as mock_find_files, \
            patch('django_spellbook.management.commands.spellbook_md.setup_directory_structure') as mock_setup, \
            patch('django_spellbook.management.commands.spellbook_md.MarkdownProcessor') as mock_processor_class:
            
            # Setup mocks
            mock_validate.return_value = (['/path1', '/path2'], ['app1', 'app2'])
            
            # Track find_files calls to return different values
            find_files_calls = 0
            def find_files_side_effect(path):
                nonlocal find_files_calls
                result = []
                if find_files_calls == 0:
                    result = [('/path1', 'file1.md'), ('/path1', 'file2.md')]
                # Second call returns empty list (no files found)
                find_files_calls += 1
                return result
            
            mock_find_files.side_effect = find_files_side_effect
            
            # Only first setup will be called
            mock_setup.return_value = ('/app1/path', '/app1/templates')
            
            # Only first processor is created
            mock_processor = MagicMock()
            mock_processor.build_toc.return_value = {}
            mock_processor.process_file.side_effect = [MagicMock(), MagicMock()]
            mock_processor_class.return_value = mock_processor
            
            # Run the command
            out = StringIO()
            call_command('spellbook_md', stdout=out)
            
            # Check output
            output = out.getvalue()
            self.assertIn('Found 2 markdown files to process', output)  # First source
            self.assertIn('Successfully processed 2 files for app1', output)
            self.assertIn('Error processing pair', output)  # Error for second pair
            self.assertIn('No markdown files found', output)  # Specific error message
            self.assertIn('1 of 2 pairs processed successfully', output)  # Summary
        
    @patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings')
    def test_configuration_error(self, mock_validate):
        """Test handling of configuration errors."""
        # Setup mock to raise error
        mock_validate.side_effect = ConfigurationError("Missing required settings")
        
        # Run command
        out = StringIO()
        with self.assertRaises(ConfigurationError):
            call_command('spellbook_md', stdout=out)
        
        # Check output
        output = out.getvalue()
        self.assertIn('Configuration error', output)
        self.assertIn('Please check your Django settings.py file', output)


# django_spellbook/tests/test_generator_utils.py

import unittest
from unittest.mock import patch, mock_open, MagicMock
import os
from pathlib import Path
from django.test import TestCase
from django.core.management.base import CommandError

from django_spellbook.management.commands.processing.generator_utils import (
    remove_leading_dash,
    get_clean_url,
    generate_view_name,
    get_template_path,
    get_spellbook_dir,
    create_file_if_not_exists,
    write_file
)

class TestGeneratorUtils(TestCase):
    
    def test_remove_leading_dash(self):
        """Test removing leading dashes from text."""
        self.assertEqual(remove_leading_dash("--test"), "test")
        self.assertEqual(remove_leading_dash("test"), "test")
        self.assertEqual(remove_leading_dash(""), "")
        self.assertEqual(remove_leading_dash("-"), "")
        
    def test_get_clean_url(self):
        """Test cleaning URL by removing leading dashes."""
        self.assertEqual(get_clean_url("--test/--page"), "test/page")
        self.assertEqual(get_clean_url("test/page"), "test/page")
        self.assertEqual(get_clean_url("--"), "")
        self.assertEqual(get_clean_url(""), "")
        
    def test_generate_view_name(self):
        """Test generating valid Python identifier for view name from URL pattern."""
        self.assertEqual(generate_view_name("test/page"), "test_page")
        self.assertEqual(generate_view_name("--test/--page"), "test_page")
        self.assertEqual(generate_view_name("test-page"), "test_page")
        self.assertEqual(generate_view_name("test_page"), "test_page")
        
    def test_get_template_path(self):
        """Test generating template path from content app and relative URL."""
        self.assertEqual(
            get_template_path("app", "test/page"), 
            os.path.join("app", "spellbook_md", "test/page.html")
        )
        
    @patch('os.path.abspath')
    @patch('os.path.dirname')
    def test_get_spellbook_dir(self, mock_dirname, mock_abspath):
        """Test getting django_spellbook base directory."""
        mock_dirname.return_value = "/fake/path"
        mock_abspath.return_value = "/absolute/fake/path"
        
        result = get_spellbook_dir()
        
        self.assertEqual(result, "/absolute/fake/path")
        
    @patch('os.path.exists')
    @patch('os.makedirs')
    @patch('builtins.open', new_callable=mock_open)
    def test_create_file_if_not_exists(self, mock_file, mock_makedirs, mock_exists):
        """Test creating a file when it doesn't exist."""
        mock_exists.return_value = False
        
        create_file_if_not_exists("/path/file.py", "content")
        
        mock_makedirs.assert_called_once_with(os.path.dirname("/path/file.py"), exist_ok=True)
        mock_file.assert_called_once_with("/path/file.py", 'w')
        mock_file().write.assert_called_once_with("content")
        
    @patch('os.path.exists')
    @patch('os.makedirs')
    @patch('builtins.open')
    def test_create_file_if_not_exists_error(self, mock_open, mock_makedirs, mock_exists):
        """Test handling error when creating a file."""
        mock_exists.return_value = False
        mock_open.side_effect = IOError("Test error")
        
        with self.assertRaises(CommandError) as context:
            create_file_if_not_exists("/path/file.py", "content")
            
        self.assertIn("Failed to create", str(context.exception))
        
    @patch('os.makedirs')
    @patch('builtins.open', new_callable=mock_open)
    def test_write_file(self, mock_file, mock_makedirs):
        """Test writing content to a file."""
        write_file("/path/file.py", "content")
        
        mock_makedirs.assert_called_once_with(os.path.dirname("/path/file.py"), exist_ok=True)
        mock_file.assert_called_once_with("/path/file.py", 'w')
        mock_file().write.assert_called_once_with("content")


# django_spellbook/tests/test_file_writer.py

import unittest
from unittest.mock import patch, mock_open, MagicMock
import os
from django.test import TestCase
from django.core.management.base import CommandError

from django_spellbook.management.commands.processing.file_writer import FileWriter

class TestFileWriter(TestCase):
    
    @patch('django_spellbook.management.commands.processing.file_writer.get_spellbook_dir')
    def setUp(self, mock_get_dir):
        """Set up the test environment."""
        mock_get_dir.return_value = "/spellbook/dir"
        
        # Patch file operations to avoid actual file system access
        with patch('django_spellbook.management.commands.processing.file_writer.create_file_if_not_exists'):
            with patch('django_spellbook.management.commands.processing.file_writer.write_file'):
                with patch('os.path.exists', return_value=False):
                    self.file_writer = FileWriter("test_app")
        
    def test_initialization(self):
        """Test initialization of FileWriter."""
        self.assertEqual(self.file_writer.content_app, "test_app")
        self.assertEqual(self.file_writer.spellbook_dir, "/spellbook/dir")
        self.assertEqual(self.file_writer.urls_module, "urls_test_app")
        self.assertEqual(self.file_writer.views_module, "views_test_app")
        
    @patch('django_spellbook.management.commands.processing.file_writer.create_file_if_not_exists')
    def test_ensure_urls_views_files(self, mock_create_file):
        """Test ensuring app-specific URLs and views files exist."""
        self.file_writer._ensure_urls_views_files()
        
        # Should create three files
        self.assertEqual(mock_create_file.call_count, 3)
        
        # Check calls to create_file_if_not_exists
        urls_file_path = os.path.join(self.file_writer.spellbook_dir, "urls_test_app.py")
        views_file_path = os.path.join(self.file_writer.spellbook_dir, "views_test_app.py")
        main_urls_path = os.path.join(self.file_writer.spellbook_dir, "urls.py")
        
        mock_create_file.assert_any_call(urls_file_path, unittest.mock.ANY)
        mock_create_file.assert_any_call(views_file_path, unittest.mock.ANY)
        mock_create_file.assert_any_call(main_urls_path, unittest.mock.ANY)
        
    @patch('os.path.exists')
    @patch('builtins.open', new_callable=mock_open)
    @patch('django_spellbook.management.commands.processing.file_writer.write_file')
    def test_update_main_urls_file(self, mock_write, mock_file, mock_exists):
        """Test updating main urls.py with app includes."""
        mock_exists.return_value = True
        mock_file.return_value.read.return_value = "urlpatterns = []"
        
        self.file_writer._update_main_urls_file()
        
        # Should call write_file for main urls.py
        main_urls_path = os.path.join(self.file_writer.spellbook_dir, "urls.py")
        mock_write.assert_called_once_with(main_urls_path, unittest.mock.ANY)
        
        # Check content
        content = mock_write.call_args[0][1]
        self.assertIn("path('test_app/', include('django_spellbook.urls_test_app'))", content)
        
    @patch('django_spellbook.management.commands.processing.file_writer.write_file')
    def test_write_urls_file(self, mock_write_file):
        """Test writing URL patterns to app-specific urls.py file."""
        url_patterns = [
            "path('test/', test_view, name='test')",
            "path('another/', another_view, name='another')"
        ]
        
        self.file_writer.write_urls_file(url_patterns)
        
        # Should write to app-specific urls.py
        urls_file_path = os.path.join(self.file_writer.spellbook_dir, "urls_test_app.py")
        mock_write_file.assert_called_once_with(urls_file_path, unittest.mock.ANY)
        
        # Check content
        content = mock_write_file.call_args[0][1]
        self.assertIn("app_name = 'test_app'", content)
        self.assertIn("path('test/', test_view, name='test')", content)
        
    @patch('django_spellbook.management.commands.processing.file_writer.write_file')
    def test_write_views_file(self, mock_write_file):
        """Test writing view functions to app-specific views.py file."""
        view_functions = [
            "def test_view(request):\n    return render(request, 'test.html', {})"
        ]
        toc = {"test": {"title": "Test"}}
        
        self.file_writer.write_views_file(view_functions, toc)
        
        # Should write to app-specific views.py
        views_file_path = os.path.join(self.file_writer.spellbook_dir, "views_test_app.py")
        mock_write_file.assert_called_once_with(views_file_path, unittest.mock.ANY)
        
        
import unittest
from unittest.mock import patch, mock_open, MagicMock
import os
from django.test import TestCase
from django.core.management.base import CommandError

from django_spellbook.management.commands.processing.file_writer import FileWriter

class TestFileWriterExceptions(TestCase):
    """Test exception handling in FileWriter class"""
    
    @patch('django_spellbook.management.commands.processing.file_writer.get_spellbook_dir')
    def setUp(self, mock_get_dir):
        """Set up the test environment."""
        mock_get_dir.return_value = "/spellbook/dir"
        
        # Patch initialization methods to avoid file operations during setup
        with patch.object(FileWriter, '_ensure_urls_views_files'), patch.object(FileWriter, '_update_main_urls_file'):
            self.file_writer = FileWriter("test_app")
    
    @patch('django_spellbook.management.commands.processing.file_writer.create_file_if_not_exists')
    def test_ensure_urls_views_files_exception(self, mock_create_file):
        """Test exception handling when creating initial files fails."""
        # Setup create_file_if_not_exists to raise an exception
        mock_create_file.side_effect = CommandError("Failed to create test file")
        
        # Should propagate the exception
        with self.assertRaises(CommandError) as context:
            self.file_writer._ensure_urls_views_files()
            
        self.assertEqual(str(context.exception), "Failed to create test file")
    
    @patch('os.path.exists')
    @patch('builtins.open')
    def test_update_main_urls_file_read_exception(self, mock_open, mock_exists):
        """Test handling of exception when reading the main urls.py file."""
        mock_exists.return_value = True
        mock_open.side_effect = IOError("Test IO error")
        
        # The method should catch the exception and log it
        with patch('django_spellbook.management.commands.processing.file_writer.logger') as mock_logger:
            with patch('django_spellbook.management.commands.processing.file_writer.write_file'):
                self.file_writer._update_main_urls_file()
                
                # Verify logger.error was called
                mock_logger.error.assert_called_once()
                self.assertIn("Error reading urls.py", mock_logger.error.call_args[0][0])
    
    @patch('django_spellbook.management.commands.processing.file_writer.write_file')
    def test_update_main_urls_file_write_exception(self, mock_write):
        """Test handling of exception when writing to main urls.py file."""
        mock_write.side_effect = CommandError("Failed to write file")
        
        # Should propagate the exception
        with self.assertRaises(CommandError) as context:
            self.file_writer._update_main_urls_file()
            
        self.assertEqual(str(context.exception), "Failed to write file")
    
    @patch('django_spellbook.management.commands.processing.file_writer.write_file')
    def test_write_urls_file_exception(self, mock_write):
        """Test exception handling when writing URLs file."""
        mock_write.side_effect = CommandError("Failed to write file")
        
        # Should raise CommandError with appropriate message
        with self.assertRaises(CommandError) as context:
            self.file_writer.write_urls_file(["path('test/', test_view, name='test')"])
            
        self.assertIn("Failed to write URLs file", str(context.exception))
    
    @patch('django_spellbook.management.commands.processing.file_writer.write_file')
    def test_write_views_file_exception(self, mock_write):
        """Test exception handling when writing views file."""
        mock_write.side_effect = CommandError("Failed to write file")
        
        # Should raise CommandError with appropriate message
        with self.assertRaises(CommandError) as context:
            self.file_writer.write_views_file(["def test_view(request): pass"], {})
            
        self.assertIn("Failed to write views file", str(context.exception))
    
    def test_urls_content_generation_exception(self):
        """Test exception handling with invalid URL patterns."""
        # Using a type that can't be joined
        invalid_url_patterns = [None, 123]
        
        with self.assertRaises(Exception):
            self.file_writer.write_urls_file(invalid_url_patterns)
    
    def test_views_content_generation_exception(self):
        """Test exception handling with invalid view functions."""
        # Using objects that can't be joined
        invalid_view_functions = [None, 123]
        
        with self.assertRaises(Exception):
            self.file_writer.write_views_file(invalid_view_functions, {})
            
    def test_error_with_toc_serialization(self):
        """Test exception handling when TOC can't be serialized."""
        # Create a TOC that can't be converted to string representation
        class NonSerializableObject:
            def __str__(self):
                raise ValueError("Cannot convert to string")
                
        invalid_toc = {"problem": NonSerializableObject()}
        
        with self.assertRaises(Exception):
            self.file_writer.write_views_file(["def test_view(request): pass"], invalid_toc)


# django_spellbook/tests/test_spellbook_md_command.py

import unittest
from unittest.mock import patch, Mock, call, MagicMock
from pathlib import Path
from io import StringIO
from django.core.management.base import CommandError
from django.test import TestCase, override_settings
from django_spellbook.management.commands.spellbook_md import Command
from django.core.management import call_command

class TestSpellbookMDCommand(TestCase):
    def setUp(self):
        """Set up test environment"""
        self.command = Command()
        self.command.stdout = Mock()  # Mock stdout for testing output

    @patch('django_spellbook.management.commands.spellbook_md.discover_blocks')
    @patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings')
    @patch('django_spellbook.management.commands.spellbook_md.Command._process_source_destination_pair')
    def test_handle_basic_execution(self, mock_process_pair, mock_validate, mock_discover):
        """Test basic successful execution of the command"""
        # Set up mocks
        mock_validate.return_value = (['/test/path'], ['test_app'])
        
        # Execute command
        self.command.handle()
        
        # Verify blocks were discovered
        mock_discover.assert_called_once_with(self.command.stdout)
        
        # Verify settings were validated
        mock_validate.assert_called_once()
        
        # Verify processing was called with correct arguments
        mock_process_pair.assert_called_once_with('/test/path', 'test_app')
        
        # Verify success message was output
        self.command.stdout.write.assert_any_call(
            self.command.style.SUCCESS(
                "Processing source-destination pair 1/1: /test/path → test_app"
            )
        )

    @patch('django_spellbook.management.commands.spellbook_md.discover_blocks')
    @patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings')
    @patch('django_spellbook.management.commands.spellbook_md.Command._process_source_destination_pair')
    def test_handle_multiple_pairs(self, mock_process_pair, mock_validate, mock_discover):
        """Test handling of multiple source-destination pairs"""
        # Set up mocks
        mock_validate.return_value = (['/path1', '/path2'], ['app1', 'app2'])
        
        # Execute command
        self.command.handle()
        
        # Verify processing was called twice with correct arguments
        expected_calls = [
            call('/path1', 'app1'),
            call('/path2', 'app2')
        ]
        self.assertEqual(mock_process_pair.call_count, 2)
        mock_process_pair.assert_has_calls(expected_calls)
        
        # Verify success messages
        self.command.stdout.write.assert_any_call(
            self.command.style.SUCCESS(
                "Processing source-destination pair 1/2: /path1 → app1"
            )
        )
        self.command.stdout.write.assert_any_call(
            self.command.style.SUCCESS(
                "Processing source-destination pair 2/2: /path2 → app2"
            )
        )

    # In test_handle_error_single_pair
    @patch('django_spellbook.management.commands.spellbook_md.discover_blocks')
    @patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings')
    @patch('django_spellbook.management.commands.spellbook_md.Command._process_source_destination_pair')
    def test_handle_error_single_pair(self, mock_process_pair, mock_validate, mock_discover):
        """Test error handling with a single pair (should raise)"""
        # Set up mocks
        mock_validate.return_value = (['/test/path'], ['test_app'])
        
        # Set up error
        test_error = CommandError("Test processing error")
        mock_process_pair.side_effect = test_error
        
        # Execute command - should raise the error
        with self.assertRaises(CommandError) as context:
            self.command.handle()
        
        # Verify it's the same error
        self.assertEqual(str(context.exception), "Test processing error")
        
        # Verify error message was output - check for substring instead of exact match
        error_logged = False
        for call_args in self.command.stdout.write.call_args_list:
            args = call_args[0]
            if len(args) > 0 and isinstance(args[0], str) and "Error processing pair" in args[0] and "Test processing error" in args[0]:
                error_logged = True
                break
        
        self.assertTrue(error_logged, "Error message was not properly logged")


    @patch('django_spellbook.management.commands.spellbook_md.discover_blocks')
    @patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings')
    @patch('django_spellbook.management.commands.spellbook_md.Command._process_source_destination_pair')
    def test_handle_error_multiple_pairs(self, mock_process_pair, mock_validate, mock_discover):
        """Test error handling with multiple pairs (should continue)"""
        # Set up mocks
        mock_validate.return_value = (['/path1', '/path2', '/path3'], ['app1', 'app2', 'app3'])
        
        # Set up error for the second pair
        def side_effect(path, app):
            if path == '/path2':
                raise CommandError("Test processing error")
            return 1  # Return an integer count on success (not None)
        
        mock_process_pair.side_effect = side_effect
        
        # Execute command
        self.command.handle()
        
        # Verify all pairs were attempted
        expected_calls = [
            call('/path1', 'app1'),
            call('/path2', 'app2'),
            call('/path3', 'app3')
        ]
        self.assertEqual(mock_process_pair.call_count, 3)
        mock_process_pair.assert_has_calls(expected_calls)
        
        # Verify error message and continue message - use substring checks
        error_logged = False
        continue_logged = False
        
        for call_args in self.command.stdout.write.call_args_list:
            args = call_args[0]
            if len(args) > 0 and isinstance(args[0], str):
                if "Error processing pair" in args[0] and "Test processing error" in args[0]:
                    error_logged = True
                elif "Continuing with next pair" in args[0]:
                    continue_logged = True
        
        self.assertTrue(error_logged, "Error message was not logged")
        self.assertTrue(continue_logged, "Continue message was not logged")
    
    @patch('django_spellbook.management.commands.spellbook_md.find_markdown_files')
    def test_process_pair_no_files(self, mock_find_files):
        """Test error when no markdown files are found"""
        # Set up mock
        mock_find_files.return_value = []
        
        # Execute and test
        with self.assertRaises(CommandError) as context:
            self.command._process_source_destination_pair('/test/path', 'test_app')
        
        self.assertIn("No markdown files found", str(context.exception))
        mock_find_files.assert_called_once_with('/test/path')

    @patch('django_spellbook.management.commands.spellbook_md.find_markdown_files')
    @patch('django_spellbook.management.commands.spellbook_md.setup_directory_structure')
    @patch('django_spellbook.management.commands.spellbook_md.MarkdownProcessor')
    def test_process_pair_successful(self, mock_processor_class, mock_setup, mock_find_files):
        """Test successful processing of a source-destination pair"""
        # Set up mocks
        mock_find_files.return_value = [('/test/path', 'file1.md'), ('/test/path', 'file2.md')]
        mock_setup.return_value = ('/content/dir/path', '/template/dir')
        
        # Create mock processor instance
        mock_processor = Mock()
        mock_processor.build_toc.return_value = {'toc': 'data'}
        
        # Create mock processed files
        mock_processed_file1 = Mock()
        mock_processed_file2 = Mock()
        mock_processor.process_file.side_effect = [mock_processed_file1, mock_processed_file2]
        
        mock_processor_class.return_value = mock_processor
        
        # Execute
        self.command._process_source_destination_pair('/test/path', 'test_app')
        
        # Verify processor was initialized correctly
        mock_processor_class.assert_called_once_with(
            content_app='test_app',
            source_path='/test/path',
            content_dir_path='/content/dir/path',
            template_dir='/template/dir'
        )
        
        # Verify TOC was built
        mock_processor.build_toc.assert_called_once()
        
        # Verify files were processed
        self.assertEqual(mock_processor.process_file.call_count, 2)
        mock_processor.process_file.assert_has_calls([
            call('/test/path', 'file1.md', {'toc': 'data'}),
            call('/test/path', 'file2.md', {'toc': 'data'})
        ])
        
        # Verify URLs and views were generated
        mock_processor.generate_urls_and_views.assert_called_once_with(
            [mock_processed_file1, mock_processed_file2], 
            {'toc': 'data'}
        )
        
        # Verify output messages
        self.command.stdout.write.assert_any_call("Found 2 markdown files to process")
        self.command.stdout.write.assert_any_call("Processing file 1/2: file1.md")
        self.command.stdout.write.assert_any_call("Processing file 2/2: file2.md")
        self.command.stdout.write.assert_any_call("Generating URLs and views...")
        self.command.stdout.write.assert_any_call(
            self.command.style.SUCCESS(
                "Successfully processed 2 files for test_app"
            )
        )

    @patch('django_spellbook.management.commands.spellbook_md.find_markdown_files')
    @patch('django_spellbook.management.commands.spellbook_md.setup_directory_structure')
    @patch('django_spellbook.management.commands.spellbook_md.MarkdownProcessor')
    def test_process_pair_file_processing_error(self, mock_processor_class, mock_setup, mock_find_files):
        """Test error handling when file processing fails"""
        # Set up mocks
        mock_find_files.return_value = [('/test/path', 'file1.md'), ('/test/path', 'file2.md')]
        mock_setup.return_value = ('/content/dir/path', '/template/dir')
        
        # Create mock processor instance
        mock_processor = Mock()
        mock_processor.build_toc.return_value = {'toc': 'data'}
        # Make process_file return None (indicating failure) for all files
        mock_processor.process_file.return_value = None
        mock_processor_class.return_value = mock_processor
        
        # Execute and test
        with self.assertRaises(CommandError) as context:
            self.command._process_source_destination_pair('/test/path', 'test_app')
        
        self.assertIn("No markdown files were processed successfully", str(context.exception))
        
        # Verify files were attempted to be processed
        self.assertEqual(mock_processor.process_file.call_count, 2)
        
        # Verify URLs and views were not generated
        mock_processor.generate_urls_and_views.assert_not_called()

    @patch('django_spellbook.management.commands.spellbook_md.find_markdown_files')
    @patch('django_spellbook.management.commands.spellbook_md.setup_directory_structure')
    @patch('django_spellbook.management.commands.spellbook_md.MarkdownProcessor')
    def test_process_pair_partial_file_processing_success(self, mock_processor_class, mock_setup, mock_find_files):
        """Test handling when some files process successfully and others fail"""
        # Set up mocks
        mock_find_files.return_value = [
            ('/test/path', 'file1.md'), 
            ('/test/path', 'file2.md'), 
            ('/test/path', 'file3.md')
        ]
        mock_setup.return_value = ('/content/dir/path', '/template/dir')
        
        # Create mock processor instance
        mock_processor = Mock()
        mock_processor.build_toc.return_value = {'toc': 'data'}
        
        # Create mock processed file (only the second file succeeds)
        mock_processed_file = Mock()
        mock_processor.process_file.side_effect = [None, mock_processed_file, None]
        mock_processor_class.return_value = mock_processor
        
        # Execute
        self.command._process_source_destination_pair('/test/path', 'test_app')
        
        # Verify all files were attempted
        self.assertEqual(mock_processor.process_file.call_count, 3)
        
        # Verify URLs and views were generated with only the successful file
        mock_processor.generate_urls_and_views.assert_called_once_with(
            [mock_processed_file], 
            {'toc': 'data'}
        )
        
        # Verify success message includes correct count
        self.command.stdout.write.assert_any_call(
            self.command.style.SUCCESS(
                "Successfully processed 1 files for test_app"
            )
        )

    # In test_handle_overall_error
    @patch('django_spellbook.management.commands.spellbook_md.discover_blocks')
    @patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings')
    def test_handle_overall_error(self, mock_validate, mock_discover):
        """Test error handling in the overall handle method"""
        # Set up an error in validation
        mock_validate.side_effect = CommandError("Invalid settings")
        
        # Execute command - should raise the error
        with self.assertRaises(CommandError) as context:
            self.command.handle()
        
        # Verify it's the same error
        self.assertIn("Invalid settings", str(context.exception))
        
        # Verify error message was output - check for substring instead of exact match
        error_logged = False
        for call_args in self.command.stdout.write.call_args_list:
            args = call_args[0]
            if len(args) > 0 and isinstance(args[0], str) and "Command failed" in args[0] and "Invalid settings" in args[0]:
                error_logged = True
                break
        
        self.assertTrue(error_logged, "Command failed message was not properly logged")
            
# django_spellbook/tests/test_spellbook_md_exceptions.py

import unittest
from unittest.mock import patch, Mock, call
from pathlib import Path
from django.test import TestCase, override_settings

from django_spellbook.management.commands.spellbook_md import (
    Command, 
    ConfigurationError,
    ContentDiscoveryError,
    ProcessingError,
    OutputGenerationError
)

class TestSpellbookMDExceptions(TestCase):
    def setUp(self):
        """Set up test environment"""
        self.command = Command()
        self.command.stdout = Mock()
    
    @patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings')
    def test_configuration_error(self, mock_validate):
        """Test error handling for configuration errors"""
        # Simulate a configuration error
        mock_validate.side_effect = Exception("Missing required settings")
        
        # Execute command - should raise the error
        with self.assertRaises(ConfigurationError) as context:
            self.command.handle()
        
        # Verify error message
        self.assertIn("Settings validation failed", str(context.exception))
        
        # Verify advice was logged
        advice_message_found = False
        for call in self.command.stdout.write.call_args_list:
            args = call[0]
            if isinstance(args[0], str) and "check your Django settings.py" in args[0]:
                advice_message_found = True
                break
        
        self.assertTrue(advice_message_found, "Advice message was not logged")
    
    @patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings')
    @patch('django_spellbook.management.commands.spellbook_md.find_markdown_files')
    def test_content_discovery_error_no_files(self, mock_find_files, mock_validate):
        """Test error handling when no markdown files are found"""
        # Setup mocks
        mock_validate.return_value = (['/test/path'], ['test_app'])
        mock_find_files.return_value = []
        
        # Execute command - should raise the error
        with self.assertRaises(ContentDiscoveryError) as context:
            self.command.handle()
        
        # Verify error message
        self.assertIn("No markdown files found", str(context.exception))
        
        # Verify advice was logged
        advice_message_found = False
        for call in self.command.stdout.write.call_args_list:
            args = call[0]
            if isinstance(args[0], str) and "Make sure the directory exists" in args[0]:
                advice_message_found = True
                break
        
        self.assertTrue(advice_message_found, "Advice message was not logged")
    
    @patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings')
    @patch('django_spellbook.management.commands.spellbook_md.find_markdown_files')
    def test_content_discovery_error_exception(self, mock_find_files, mock_validate):
        """Test error handling when file discovery throws an exception"""
        # Setup mocks
        mock_validate.return_value = (['/test/path'], ['test_app'])
        mock_find_files.side_effect = OSError("Permission denied")
        
        # Execute command - should raise the error
        with self.assertRaises(ContentDiscoveryError) as context:
            self.command.handle()
        
        # Verify error message
        self.assertIn("Failed to discover markdown files", str(context.exception))
        self.assertIn("Permission denied", str(context.exception))
    
    @patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings')
    @patch('django_spellbook.management.commands.spellbook_md.find_markdown_files')
    @patch('django_spellbook.management.commands.spellbook_md.setup_directory_structure')
    def test_configuration_error_directory_setup(self, mock_setup, mock_find_files, mock_validate):
        """Test error handling for directory setup failures"""
        # Setup mocks
        mock_validate.return_value = (['/test/path'], ['test_app'])
        mock_find_files.return_value = [('/test/path', 'test.md')]
        mock_setup.side_effect = OSError("Permission denied")
        
        # Execute command - should raise the error
        with self.assertRaises(ConfigurationError) as context:
            self.command.handle()
        
        # Verify error message
        self.assertIn("Failed to set up directory structure", str(context.exception))
        self.assertIn("Permission denied", str(context.exception))
        
        # Verify advice was logged
        advice_message_found = False
        for call in self.command.stdout.write.call_args_list:
            args = call[0]
            if isinstance(args[0], str) and "Check that the content app exists" in args[0]:
                advice_message_found = True
                break
        
        self.assertTrue(advice_message_found, "Advice message was not logged")
    
    @patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings')
    @patch('django_spellbook.management.commands.spellbook_md.find_markdown_files')
    @patch('django_spellbook.management.commands.spellbook_md.setup_directory_structure')
    @patch('django_spellbook.management.commands.spellbook_md.MarkdownProcessor')
    def test_toc_error_continues_processing(self, mock_processor_class, mock_setup, mock_find_files, mock_validate):
        """Test error handling for TOC building failures"""
        # Setup mocks
        mock_validate.return_value = (['/test/path'], ['test_app'])
        mock_find_files.return_value = [('/test/path', 'test.md')]
        mock_setup.return_value = ('/test/path/app', '/test/path/app/templates')
        
        # Create a mock processor with a failing build_toc method
        mock_processor = Mock()
        mock_processor.build_toc.side_effect = Exception("TOC error")
        mock_processor.process_file.return_value = Mock()  # Process succeeds
        mock_processor_class.return_value = mock_processor
        
        # Execute command
        self.command.handle()
        
        # Verify warning was logged
        warning_message_found = False
        for call in self.command.stdout.write.call_args_list:
            args = call[0]
            if isinstance(args[0], str) and "Error building table of contents" in args[0]:
                warning_message_found = True
                break
        
        self.assertTrue(warning_message_found, "Warning about TOC failure was not logged")
        
        # Verify the command continued with an empty TOC
        mock_processor.process_file.assert_called_once()
    
    @patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings')
    @patch('django_spellbook.management.commands.spellbook_md.find_markdown_files')
    @patch('django_spellbook.management.commands.spellbook_md.setup_directory_structure')
    @patch('django_spellbook.management.commands.spellbook_md.MarkdownProcessor')
    def test_process_multiple_pairs_summary(self, mock_processor_class, mock_setup, mock_find_files, mock_validate):
        """Test summary report with multiple source-destination pairs"""
        # Setup mocks
        mock_validate.return_value = (['/path1', '/path2'], ['app1', 'app2'])
        
        # Mock file discovery for each pair
        mock_find_files.side_effect = [
            [('/path1', 'file1.md')],  # First pair has files
            []  # Second pair has no files (will fail)
        ]
        
        mock_setup.return_value = ('/content/path', '/template/path')
        
        # Create a successful processor for first pair
        mock_processor = Mock()
        mock_processor.build_toc.return_value = {}
        mock_processor.process_file.return_value = Mock()
        mock_processor_class.return_value = mock_processor
        
        # Execute command - second pair will fail but command should continue
        self.command.handle()
        
        # Verify summary was generated
        summary_message_found = False
        failed_pair_message_found = False
        
        for call in self.command.stdout.write.call_args_list:
            args = call[0]
            if isinstance(args[0], str) and "Processing Summary:" in args[0]:
                summary_message_found = True
            elif isinstance(args[0], str) and "1 of 2 pairs processed successfully" in args[0]:
                failed_pair_message_found = True
        
        self.assertTrue(summary_message_found, "Summary was not generated")
        self.assertTrue(failed_pair_message_found, "Failed pair message not found in summary")
    
    @patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings')
    @patch('django_spellbook.management.commands.spellbook_md.find_markdown_files')
    @patch('django_spellbook.management.commands.spellbook_md.setup_directory_structure')
    @patch('django_spellbook.management.commands.spellbook_md.MarkdownProcessor')
    def test_continue_on_error_flag(self, mock_processor_class, mock_setup, mock_find_files, mock_validate):
        """Test --continue-on-error flag allows processing to continue after file errors"""
        # Setup mocks
        mock_validate.return_value = (['/test/path'], ['test_app'])
        mock_find_files.return_value = [
            ('/test/path', 'good.md'),
            ('/test/path', 'bad.md')
        ]
        mock_setup.return_value = ('/content/path', '/template/path')
        
        # Create processor mock where one file succeeds and one fails
        mock_processor = Mock()
        mock_processor.build_toc.return_value = {}
        
        # First file succeeds, second file raises an error
        mock_processed_file = Mock()
        
        # Use side_effect as a list to return a value for the first call and raise exception for the second
        def side_effect(dirpath, filename, toc):
            if filename == 'good.md':
                return mock_processed_file
            else:
                raise Exception("Error processing file")
        
        mock_processor.process_file.side_effect = side_effect
        mock_processor.generate_urls_and_views.return_value = None
        mock_processor_class.return_value = mock_processor
        
        # Pass the continue_on_error flag using options
        # This is critical - we need to pass it as an option rather than setting the attribute
        options = {'continue_on_error': True}
        self.command.handle(**options)
        
        # Verify both files were attempted
        self.assertEqual(mock_processor.process_file.call_count, 2)
        
        # Verify URLs and views were generated with the successful file
        mock_processor.generate_urls_and_views.assert_called_once()
        
        # Verify error was logged but processing continued
        error_logged = False
        success_logged = False
        
        for call_args in self.command.stdout.write.call_args_list:
            args = call_args[0]
            if len(args) > 0 and isinstance(args[0], str):
                if "Error processing file" in args[0]:
                    error_logged = True
                elif "Successfully processed" in args[0] and "1 files" in args[0]:
                    success_logged = True
        
        self.assertTrue(error_logged, "Error message was not logged")
        self.assertTrue(success_logged, "Success message was not logged")
        
    def test_block_discovery_error(self):
        """Test handling of errors during block discovery."""
        with patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings') as mock_validate, \
            patch('django_spellbook.management.commands.spellbook_md.discover_blocks') as mock_discover, \
            patch('django_spellbook.management.commands.spellbook_md.find_markdown_files') as mock_find_files, \
            patch('django_spellbook.management.commands.spellbook_md.setup_directory_structure') as mock_setup, \
            patch('django_spellbook.management.commands.spellbook_md.MarkdownProcessor') as mock_processor_class:
            
            # Setup mocks
            mock_validate.return_value = (['/test/path'], ['test_app'])
            mock_discover.side_effect = Exception("Test discovery error")
            mock_find_files.return_value = [('/test/path', 'test.md')]
            mock_setup.return_value = ('/app/path', '/app/templates')
            
            # Setup processor mock
            mock_processor = MagicMock()
            mock_processor.build_toc.return_value = {}
            mock_processor.process_file.return_value = MagicMock()
            mock_processor_class.return_value = mock_processor
            
            # Run the command
            out = StringIO()
            call_command('spellbook_md', stdout=out)
            
            # Check output
            output = out.getvalue()
            self.assertIn('Error during block discovery: Test discovery error', output)
            self.assertIn('Processing will continue but some content may not render correctly', output)
            
            # Verify command continued processing
            self.assertIn('Successfully processed 1 files for test_app', output)

    def test_processor_initialization_error(self):
        """Test handling of errors during processor initialization."""
        with patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings') as mock_validate, \
            patch('django_spellbook.management.commands.spellbook_md.find_markdown_files') as mock_find_files, \
            patch('django_spellbook.management.commands.spellbook_md.setup_directory_structure') as mock_setup, \
            patch('django_spellbook.management.commands.spellbook_md.MarkdownProcessor') as mock_processor_class:
            
            # Setup mocks
            mock_validate.return_value = (['/test/path'], ['test_app'])
            mock_find_files.return_value = [('/test/path', 'test.md')]
            mock_setup.return_value = ('/app/path', '/app/templates')
            
            # Make processor initialization fail
            mock_processor_class.side_effect = Exception("Test processor initialization error")
            
            # Run the command - should raise a ProcessingError
            out = StringIO()
            with self.assertRaises(ProcessingError) as context:
                call_command('spellbook_md', stdout=out)
            
            # Check the exception message
            self.assertIn('Failed to initialize markdown processor', str(context.exception))
            self.assertIn('Test processor initialization error', str(context.exception))

    def test_url_view_generation_error(self):
        """Test handling of errors during URL and view generation."""
        with patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings') as mock_validate, \
            patch('django_spellbook.management.commands.spellbook_md.find_markdown_files') as mock_find_files, \
            patch('django_spellbook.management.commands.spellbook_md.setup_directory_structure') as mock_setup, \
            patch('django_spellbook.management.commands.spellbook_md.MarkdownProcessor') as mock_processor_class:
            
            # Setup mocks
            mock_validate.return_value = (['/test/path'], ['test_app'])
            mock_find_files.return_value = [('/test/path', 'test.md')]
            mock_setup.return_value = ('/app/path', '/app/templates')
            
            # Setup processor mock
            mock_processor = MagicMock()
            mock_processor.build_toc.return_value = {}
            mock_processor.process_file.return_value = MagicMock()
            
            # Make URL/view generation fail
            mock_processor.generate_urls_and_views.side_effect = Exception("Test URL generation error")
            mock_processor_class.return_value = mock_processor
            
            # Run the command - should raise an OutputGenerationError
            out = StringIO()
            with self.assertRaises(OutputGenerationError) as context:
                call_command('spellbook_md', stdout=out)
            
            # Check the exception message
            self.assertIn('Failed to generate URLs and views', str(context.exception))
            self.assertIn('Test URL generation error', str(context.exception))
            
            # Check output
            output = out.getvalue()
            self.assertIn('Error generating URLs and views', output)
            
    def test_error_stops_processing_by_default(self):
        """Test that errors stop processing by default without continue_on_error flag."""
        with patch('django_spellbook.management.commands.spellbook_md.validate_spellbook_settings') as mock_validate, \
            patch('django_spellbook.management.commands.spellbook_md.find_markdown_files') as mock_find_files, \
            patch('django_spellbook.management.commands.spellbook_md.setup_directory_structure') as mock_setup, \
            patch('django_spellbook.management.commands.spellbook_md.MarkdownProcessor') as mock_processor_class:
            
            # Setup mocks
            mock_validate.return_value = (['/test/path'], ['test_app'])
            mock_find_files.return_value = [
                ('/test/path', 'good.md'),
                ('/test/path', 'bad.md'),
                ('/test/path', 'never_processed.md')  # This file should never be processed
            ]
            mock_setup.return_value = ('/app/path', '/app/templates')
            
            # Setup processor mock
            mock_processor = MagicMock()
            mock_processor.build_toc.return_value = {}
            
            # Configure process_file to succeed for good.md but fail for bad.md
            def process_file_side_effect(dirpath, filename, toc):
                if filename == 'bad.md':
                    raise Exception("Test error in file processing")
                return MagicMock()  # Return a mock for successful file
            
            mock_processor.process_file.side_effect = process_file_side_effect
            mock_processor_class.return_value = mock_processor
            
            # Run the command WITHOUT continue-on-error flag (default behavior)
            out = StringIO()
            
            # It should raise a ProcessingError
            with self.assertRaises(ProcessingError) as context:
                call_command('spellbook_md', stdout=out)
            
            # Check the exception message
            self.assertIn('Failed to process file bad.md', str(context.exception))
            self.assertIn('Test error in file processing', str(context.exception))
            
            # Verify only the first file was processed before the error
            self.assertEqual(mock_processor.process_file.call_count, 2)  # only good.md and bad.md were attempted
            
            # Check output
            output = out.getvalue()
            self.assertIn('Error processing file bad.md', output)
            
            # generate_urls_and_views should not have been called since processing stopped at the error
            mock_processor.generate_urls_and_views.assert_not_called()


from typing import Dict, Any, Set
from django.template.loader import render_to_string
import markdown
from logging import getLogger

logger = getLogger(__name__)

class BasicSpellBlock:
    """
    Base class for spell blocks that render markdown content with specific templates.
    Now uses the custom MarkdownParser for more consistent processing.
    """

    def __init__(self, content=None, **kwargs):
        self.content = content
        self.kwargs = kwargs
        self.name = getattr(self, 'name', None)
        self.template = getattr(self, 'template', None)
        self.required_kwargs = getattr(self, 'required_kwargs', set())
        self.optional_kwargs = getattr(self, 'optional_kwargs', set())

    def get_context(self) -> Dict[str, Any]:
        """
        Get the context dictionary for template rendering.
        """
        return {
            'content': self.process_content(),
            **self.kwargs
        }

    def process_content(self) -> str:
        """
        Process markdown content using our custom MarkdownParser.
        This ensures consistent processing across the entire application.
        """
        if self.content is None:
            return ''

        from django_spellbook.markdown.parser import MarkdownParser

        parser = MarkdownParser(self.content)
        return parser.get_html()

    def render(self) -> str:
        """
        Render the block using its template and context.
        """
        if not self.template:
            raise ValueError(f"No template specified for block {self.name}")

        context = self.get_context()
        return render_to_string(self.template, context)


# django_spellbook/management/commands/spellbook_md/discovery.py

import os
import logging
from typing import List, Tuple, Optional, IO, Set
from pathlib import Path
import importlib
from functools import lru_cache

from django.apps import apps
from django.core.management.base import CommandError

from django_spellbook.blocks import SpellBlockRegistry

logger = logging.getLogger(__name__)

def log_and_write(message: str, level: str = 'info', stdout: Optional[IO] = None) -> None:
    """
    Log a message and optionally write to stdout.
    
    Args:
        message: The message to log and write
        level: The logging level (debug, info, warning, error)
        stdout: Optional output stream to write to
    """
    getattr(logger, level)(message)
    if stdout:
        stdout.write(message)

@lru_cache(maxsize=1)
def discover_blocks(stdout: Optional[IO] = None) -> int:
    """
    Discover and register spell blocks from all installed apps.
    
    This function attempts to import a 'spellblocks' module from each installed
    Django app and register any spell blocks defined there.
    
    Args:
        stdout: Optional output stream to write progress information
        
    Raises:
        CommandError: If a critical error occurs during block discovery
    
    Returns:
        Number of blocks discovered and registered
    """
    log_and_write("Starting block discovery...", stdout=stdout)
    
    for app_config in apps.get_app_configs():
        try:
            # Try to import spellblocks.py from each app
            module_path = f"{app_config.name}.spellblocks"
            log_and_write(f"Checking {module_path}...", 'debug', stdout)

            importlib.import_module(module_path)
            log_and_write(f"Discovered blocks from {app_config.name}", stdout=stdout)

        except ImportError:
            # Skip if no spellblocks.py exists
            log_and_write(f"No blocks found in {app_config.name}", 'debug', stdout)
            continue
        except Exception as e:
            log_and_write(f"Error loading blocks from {app_config.name}: {str(e)}", 'error', stdout)
            continue

    block_count = len(SpellBlockRegistry._registry)
    log_and_write(f"Block discovery complete. Found {block_count} blocks.", stdout=stdout)
    
    return block_count

def find_markdown_files(source_path: str, 
                        exclude_dirs: Optional[Set[str]] = None) -> List[Tuple[str, str]]:
    """
    Find all markdown files in the source path.
    
    Args:
        source_path: Path to the source directory (string or Path object)
        exclude_dirs: Optional set of directory names to exclude from search
    
    Returns:
        List of tuples containing (directory path, filename) for each markdown file
        
    Raises:
        FileNotFoundError: If the source path doesn't exist
        PermissionError: If there are permission issues accessing the directory
    """
    if exclude_dirs is None:
        exclude_dirs = set()
        
    # Convert to Path object for consistent handling
    path = Path(source_path)
    
    if not path.exists():
        raise FileNotFoundError(f"Source path does not exist: {path}")
    
    if not path.is_dir():
        raise NotADirectoryError(f"Source path is not a directory: {path}")
        
    markdown_files: List[Tuple[str, str]] = []
    
    try:
        for dirpath, dirnames, filenames in os.walk(str(path)):
            # Modify dirnames in-place to exclude certain directories
        
            dirnames[:] = [d for d in dirnames if d not in exclude_dirs]
            
            for filename in filenames:
                if filename.endswith('.md'):
                    markdown_files.append((dirpath, filename))
    except Exception as e:
        logger.error(f"Error finding markdown files: {str(e)}")
        raise
        
    return markdown_files


# django_spellbook/tests/test_view_generator.py

import unittest
from unittest.mock import patch, Mock
import datetime
from pathlib import Path
from django.test import TestCase

from django_spellbook.management.commands.processing.view_generator import ViewGenerator
from django_spellbook.management.commands.processing.file_processor import ProcessedFile
from django_spellbook.markdown.context import SpellbookContext

class TestViewGenerator(TestCase):
    
    def setUp(self):
        """Set up the test environment."""
        self.view_generator = ViewGenerator("test_app")
        
        # Create a mock context
        self.mock_context = Mock(spec=SpellbookContext)
        self.mock_context.__dict__ = {
            'title': 'Test Page', 
            'toc': {},
            'created_at': datetime.datetime(2023, 1, 1),
            'is_public': True,
            'tags': ['test', 'example'],
            'custom_meta': {'key': 'value'}
        }
        
        # Create a sample processed file
        self.processed_file = ProcessedFile(
            original_path=Path('/test/file.md'),
            html_content='<h1>Test</h1>',
            template_path=Path('/test/template.html'),
            relative_url='test/page',
            context=self.mock_context
        )
        
    def test_initialization(self):
        """Test initialization of ViewGenerator."""
        self.assertEqual(self.view_generator.content_app, "test_app")
        
    def test_prepare_context_dict(self):
        """Test preparing context dictionary for template rendering."""
        result = self.view_generator._prepare_context_dict(self.mock_context)
        
        # Check that toc is removed
        self.assertNotIn('toc', result)
        
        # Check other values
        self.assertEqual(result['title'], 'Test Page')
        self.assertEqual(result['is_public'], True)
        self.assertEqual(result['tags'], ['test', 'example'])
        
    def test_prepare_metadata(self):
        """Test preparing metadata dictionary for a file."""
        result = self.view_generator._prepare_metadata(self.processed_file)
        
        # Check metadata fields
        self.assertEqual(result['title'], 'Test Page')
        self.assertEqual(result['is_public'], True)
        self.assertEqual(result['tags'], ['test', 'example'])
        self.assertEqual(result['namespace'], 'test_app')
        self.assertEqual(result['url_name'], 'test_page')
        self.assertEqual(result['namespaced_url'], 'test_app:test_page')
        
    @patch('django_spellbook.management.commands.processing.view_generator.generate_view_name')
    @patch('django_spellbook.management.commands.processing.view_generator.get_template_path')
    @patch('django_spellbook.management.commands.processing.view_generator.get_clean_url')
    def test_generate_view_function(self, mock_clean_url, mock_template_path, mock_view_name):
        """Test generating view function for a single file."""
        # Configure mocks
        mock_clean_url.return_value = "test/page"
        mock_template_path.return_value = "test_app/spellbook_md/test/page.html"
        mock_view_name.return_value = "test_page"
        
        # Mock _prepare_metadata and _prepare_context_dict
        with patch.object(self.view_generator, '_prepare_metadata') as mock_prepare_metadata:
            with patch.object(self.view_generator, '_prepare_context_dict') as mock_prepare_context:
                # Configure mocks
                mock_prepare_metadata.return_value = {'title': 'Test', 'url_path': 'test/page'}
                mock_prepare_context.return_value = {'title': 'Test'}
                
                # Call method
                view_function = self.view_generator._generate_view_function(self.processed_file)
                
                # Check result is a string containing a view function
                self.assertIsInstance(view_function, str)
                self.assertTrue(view_function.startswith('\ndef test_page(request):'))
                self.assertIn("context = {'title': 'Test'}", view_function)
                self.assertIn("context['toc'] = TOC", view_function)
                self.assertIn("return render(request,", view_function)
                
import unittest
from unittest.mock import patch, Mock, MagicMock
import datetime
from pathlib import Path
from django.test import TestCase
from django.core.management.base import CommandError

from django_spellbook.management.commands.processing.view_generator import ViewGenerator
from django_spellbook.management.commands.processing.file_processor import ProcessedFile
from django_spellbook.markdown.context import SpellbookContext

class TestViewGeneratorErrors(TestCase):
    
    def test_init_with_invalid_content_app(self):
        """Test initialization with invalid content_app parameter."""
        # Test with empty string
        with self.assertRaises(CommandError) as context:
            ViewGenerator("")
        self.assertIn("Content app name must be a non-empty string", str(context.exception))
        
        # Test with None
        with self.assertRaises(CommandError) as context:
            ViewGenerator(None)
        self.assertIn("Content app name must be a non-empty string", str(context.exception))
        
        # Test with non-string
        with self.assertRaises(CommandError) as context:
            ViewGenerator(123)
        self.assertIn("Content app name must be a non-empty string", str(context.exception))
    
    def test_generate_view_functions_with_empty_list(self):
        """Test generate_view_functions with an empty list."""
        view_generator = ViewGenerator("test_app")
        
        # Test with empty list - should return empty list
        with patch('django_spellbook.management.commands.processing.view_generator.logger') as mock_logger:
            result = view_generator.generate_view_functions([])
            
            self.assertEqual(result, [])
            mock_logger.warning.assert_called_once()
            self.assertIn("No processed files provided", mock_logger.warning.call_args[0][0])
    
    def test_generate_view_functions_with_invalid_items(self):
        """Test generate_view_functions with list containing invalid items."""
        view_generator = ViewGenerator("test_app")
        
        # Create a list with non-ProcessedFile items
        invalid_items = [
            None,
            "not a processed file",
            123,
            {}
        ]
        
        with patch('django_spellbook.management.commands.processing.view_generator.logger') as mock_logger:
            result = view_generator.generate_view_functions(invalid_items)
            
            self.assertEqual(result, [])
            self.assertEqual(mock_logger.warning.call_count, 5)  # 1 for empty result + 4 for invalid items
    
    def test_generate_view_function_with_missing_attributes(self):
        """Test _generate_view_function with ProcessedFile missing required attributes."""
        view_generator = ViewGenerator("test_app")
        
        # Create ProcessedFile without relative_url
        missing_url = Mock(spec=ProcessedFile)
        missing_url.context = Mock(spec=SpellbookContext)
        
        # Use delattr to remove the attribute
        delattr(missing_url, 'relative_url')
        
        with patch('django_spellbook.management.commands.processing.view_generator.logger') as mock_logger:
            with self.assertRaises(CommandError) as context:
                view_generator._generate_view_function(missing_url)
                
            self.assertIn("Failed to generate view function", str(context.exception))
            mock_logger.error.assert_called_once()
    
    def test_generate_view_function_with_empty_relative_url(self):
        """Test _generate_view_function with empty relative_url."""
        view_generator = ViewGenerator("test_app")
        
        # Mock generate_view_name to return empty string
        with patch('django_spellbook.management.commands.processing.view_generator.generate_view_name', 
                   return_value=""):
            
            processed_file = Mock(spec=ProcessedFile)
            processed_file.relative_url = "test/page"
            processed_file.context = Mock(spec=SpellbookContext)
            
            with patch('django_spellbook.management.commands.processing.view_generator.logger') as mock_logger:
                with self.assertRaises(CommandError) as context:
                    view_generator._generate_view_function(processed_file)
                    
                self.assertIn("Could not generate valid view name", str(context.exception))
                mock_logger.error.assert_called_once()
    
    def test_prepare_metadata_with_invalid_context(self):
        """Test _prepare_metadata with invalid context."""
        view_generator = ViewGenerator("test_app")
        
        # Create a ProcessedFile with invalid context
        processed_file = Mock(spec=ProcessedFile)
        processed_file.relative_url = "test/page"
        processed_file.context = None
        
        # We don't expect an error to be logged in this case based on current implementation
        metadata = view_generator._prepare_metadata(processed_file)
        
        # Should return minimal metadata
        self.assertIn('title', metadata)
        self.assertIn('url_path', metadata)
        self.assertIn('namespace', metadata)
    
    def test_prepare_context_dict_with_none_context(self):
        """Test _prepare_context_dict with None context."""
        view_generator = ViewGenerator("test_app")
        
        with patch('django_spellbook.management.commands.processing.view_generator.logger') as mock_logger:
            result = view_generator._prepare_context_dict(None)
            
            self.assertEqual(result, {})
            mock_logger.warning.assert_called_once()
            self.assertIn("Context is None", mock_logger.warning.call_args[0][0])
    
    def test_prepare_context_dict_with_context_raising_exception(self):
        """Test _prepare_context_dict with context that raises exception."""
        view_generator = ViewGenerator("test_app")
        
        # Create a context that raises exception when __dict__ is accessed
        class ProblemContext:
            @property
            def __dict__(self):
                raise Exception("Test exception")
        
        problematic_context = ProblemContext()
        
        with patch('django_spellbook.management.commands.processing.view_generator.logger') as mock_logger:
            result = view_generator._prepare_context_dict(problematic_context)
            
            self.assertEqual(result, {})
            mock_logger.error.assert_called_once()
            self.assertIn("Error preparing context", mock_logger.error.call_args[0][0])
    
    def test_safe_get_attr_with_none_object(self):
        """Test _safe_get_attr with None object."""
        view_generator = ViewGenerator("test_app")
        
        # Should return default value
        self.assertEqual(view_generator._safe_get_attr(None, 'any_attr', 'default'), 'default')
    
    def test_context_dict_with_problematic_values(self):
        """Test _prepare_context_dict with values that raise errors."""
        view_generator = ViewGenerator("test_app")
        
        # Create a datetime subclass that raises exception when repr() is called
        class ProblemDatetime(datetime.datetime):
            def __repr__(self):
                raise Exception("Cannot represent")
        
        # Create a problematic datetime value (can't just instantiate ProblemDatetime directly)
        problem_datetime = datetime.datetime(2023, 1, 1)
        problem_value = Mock(spec=datetime.datetime)
        problem_value.__repr__ = Mock(side_effect=Exception("Cannot represent"))
        
        context = Mock(spec=SpellbookContext)
        context.__dict__ = {
            'good_value': 'test',
            'problem_value': problem_value,
            'date_value': datetime.datetime(2023, 1, 1)
        }
        
        with patch('django_spellbook.management.commands.processing.view_generator.logger') as mock_logger:
            with patch('django_spellbook.management.commands.processing.view_generator.isinstance', 
                    return_value=True):  # Force all values to be treated as datetime
                result = view_generator._prepare_context_dict(context)
                
                # Should include good values but set problematic ones to None
                self.assertEqual(result['good_value'], "'test'")
                self.assertIsNone(result['problem_value'])  # Should be None due to exception handling
                self.assertTrue(isinstance(result['date_value'], str))
                mock_logger.warning.assert_called_once()
                self.assertIn("Could not process context value", mock_logger.warning.call_args[0][0])
    
    def test_metadata_representation_with_problematic_values(self):
        """Test metadata representation with values that can't be represented."""
        view_generator = ViewGenerator("test_app")
        
        # Create a class that raises exception when repr() is called
        class ProblemValue:
            def __repr__(self):
                raise Exception("Cannot represent")
        
        # Create a processed file with context
        processed_file = Mock(spec=ProcessedFile)
        processed_file.relative_url = "test/page"
        processed_file.context = Mock(spec=SpellbookContext)
        
        # Mock _prepare_metadata to return dict with problematic value
        with patch.object(view_generator, '_prepare_metadata') as mock_prepare:
            mock_prepare.return_value = {
                'good_key': 'value',
                'problem_key': ProblemValue()
            }
            
            with patch('django_spellbook.management.commands.processing.view_generator.logger') as mock_logger:
                # This should handle the exception and continue
                result = view_generator._generate_view_function(processed_file)
                
                # Should log a warning
                mock_logger.warning.assert_called_once()
                self.assertIn("Could not represent metadata value", mock_logger.warning.call_args[0][0])
                
                # Result should include the good key and a comment for the problem key
                self.assertIn("'good_key': 'value'", result)
                self.assertIn("# Error representing value", result)
    
    def test_generate_view_function_comprehensive_error_handling(self):
        """Test _generate_view_function handles various errors and provides helpful messages."""
        view_generator = ViewGenerator("test_app")
        
        # Test with various exceptions in helper methods
        with patch('django_spellbook.management.commands.processing.view_generator.generate_view_name', 
                  side_effect=Exception("View name error")):
            
            processed_file = Mock(spec=ProcessedFile)
            processed_file.relative_url = "test/page"
            processed_file.context = Mock(spec=SpellbookContext)
            
            with self.assertRaises(CommandError) as context:
                view_generator._generate_view_function(processed_file)
                
            self.assertIn("Failed to generate view function", str(context.exception))
            self.assertIn("View name error", str(context.exception))
    
    def test_generate_view_functions_error_continuation(self):
        """Test generate_view_functions continues processing after errors."""
        view_generator = ViewGenerator("test_app")
        
        # Create good and bad processed files
        good_file = Mock(spec=ProcessedFile)
        good_file.relative_url = "good/file"
        good_file.context = Mock(spec=SpellbookContext)
        
        bad_file = Mock(spec=ProcessedFile)
        bad_file.relative_url = "bad/file"
        bad_file.context = Mock(spec=SpellbookContext)
        
        # Make _generate_view_function succeed for good_file and fail for bad_file
        def mock_generate(file):
            if file == good_file:
                return "def good_file(): pass"
            else:
                raise Exception("Error in bad file")
                
        with patch.object(view_generator, '_generate_view_function', side_effect=mock_generate):
            with patch('django_spellbook.management.commands.processing.view_generator.logger') as mock_logger:
                result = view_generator.generate_view_functions([good_file, bad_file])
                
                # Should have one successful function and log error for bad file
                self.assertEqual(len(result), 1)
                self.assertEqual(result[0], "def good_file(): pass")
                mock_logger.error.assert_called_once()
                self.assertIn("Error generating view function", mock_logger.error.call_args[0][0])
    
    def test_prepare_metadata_exception_handling(self):
        """Test _prepare_metadata handles exceptions in attribute access."""
        view_generator = ViewGenerator("test_app")
        
        # Create a ProcessedFile that raises exception when relative_url is accessed
        class ExceptionFile:
            @property
            def relative_url(self):
                raise Exception("Relative URL error")
                
            # Add minimal required attributes to pass isinstance check
            def __init__(self):
                self.original_path = "test"
                self.html_content = "test"
                self.template_path = "test"
                self.context = None
        
        exception_file = ExceptionFile()
        
        with patch('django_spellbook.management.commands.processing.view_generator.logger') as mock_logger:
            # Should return minimal metadata and log the error
            with self.assertRaises(Exception):
                metadata = view_generator._prepare_metadata(exception_file)
                
    def test_generate_view_function_missing_context_attribute(self):
        """Test _generate_view_function with ProcessedFile missing context attribute."""
        view_generator = ViewGenerator("test_app")
        
        # Create ProcessedFile without context attribute
        processed_file = Mock(spec=ProcessedFile)
        processed_file.relative_url = "test/page"
        # Deliberately remove context attribute
        if hasattr(processed_file, 'context'):
            delattr(processed_file, 'context')
        
        with patch('django_spellbook.management.commands.processing.view_generator.logger') as mock_logger:
            with self.assertRaises(CommandError) as context:
                view_generator._generate_view_function(processed_file)
                
            self.assertIn("Failed to generate view function", str(context.exception))
            self.assertIn("ProcessedFile missing context attribute", str(context.exception))
            mock_logger.error.assert_called_once()

    def test_prepare_context_dict_outer_exception(self):
        """Test _prepare_context_dict with exception in the outer try-except block."""
        view_generator = ViewGenerator("test_app")
        
        # Create a normal context
        context = Mock(spec=SpellbookContext)
        
        # Force an exception in the main try block by patching hasattr
        with patch('django_spellbook.management.commands.processing.view_generator.hasattr', 
                side_effect=Exception("Error in hasattr")):
            with patch('django_spellbook.management.commands.processing.view_generator.logger') as mock_logger:
                result = view_generator._prepare_context_dict(context)
                
                self.assertEqual(result, {})
                mock_logger.error.assert_called_once()
                self.assertIn("Error preparing context dictionary", mock_logger.error.call_args[0][0])
                
    def test_context_dict_representation_check(self):
        """Test context dictionary representation check in _generate_view_function."""
        view_generator = ViewGenerator("test_app")
        
        # Create a processed file with valid attributes
        processed_file = Mock(spec=ProcessedFile)
        processed_file.relative_url = "test/page"
        processed_file.context = Mock(spec=SpellbookContext)
        
        # Create a problematic object that can't be represented
        class ProblematicObject:
            def __repr__(self):
                raise Exception("Cannot represent!")
        
        problematic_dict = {'normal': 'test', 'problem': ProblematicObject()}
        
        # Mock _prepare_context_dict to return the problematic dict
        with patch.object(view_generator, '_prepare_context_dict', return_value=problematic_dict):
            with patch.object(view_generator, '_prepare_metadata', return_value={}):
                with patch('django_spellbook.management.commands.processing.view_generator.logger') as mock_logger:
                    # Should handle the exception during repr() and use an empty context
                    result = view_generator._generate_view_function(processed_file)
                    
                    # Should log the error
                    mock_logger.error.assert_called_once()
                    self.assertIn("security validation failed", mock_logger.error.call_args[0][0])
                    
                    # Should use an empty context
                    self.assertIn("context = {}", result)

    def test_context_dict_with_dangerous_content(self):
        """Test security validation for dangerous content in context."""
        view_generator = ViewGenerator("test_app")
        
        # Create a processed file with valid attributes
        processed_file = Mock(spec=ProcessedFile)
        processed_file.relative_url = "test/page" 
        processed_file.context = Mock(spec=SpellbookContext)
        
        # Create context dictionaries with potentially dangerous content
        dangerous_contexts = [
            {'normal': 'test', 'danger': '__class__.__subclasses__()'},
            {'normal': 'test', 'danger': 'eval("import os")'},
            {'normal': 'test', 'danger': 'exec("x=1")'},
            {'normal': 'test', 'danger': 'import os; os.system("rm -rf /")'}
        ]
        
        for dangerous_dict in dangerous_contexts:
            # Mock _prepare_context_dict to return the dangerous dict
            with patch.object(view_generator, '_prepare_context_dict', return_value=dangerous_dict):
                with patch.object(view_generator, '_prepare_metadata', return_value={}):
                    with patch('django_spellbook.management.commands.processing.view_generator.logger') as mock_logger:
                        # Should detect dangerous content and use an empty context
                        result = view_generator._generate_view_function(processed_file)
                        
                        # Should log the error
                        mock_logger.error.assert_called_once()
                        self.assertIn("security validation failed", mock_logger.error.call_args[0][0])
                        self.assertIn("Potentially unsafe content", mock_logger.error.call_args[0][0])
                        
                        # Should use an empty context
                        self.assertIn("context = {}", result)
                        
                        # Reset mock
                        mock_logger.reset_mock()

    def test_context_dict_with_safe_content(self):
        """Test security validation with safe content passes."""
        view_generator = ViewGenerator("test_app")
        
        # Create a processed file with valid attributes
        processed_file = Mock(spec=ProcessedFile)
        processed_file.relative_url = "test/page"
        processed_file.context = Mock(spec=SpellbookContext)
        
        # Create a context with safe values
        safe_dict = {
            'string': 'test',
            'number': 123,
            'boolean': True,
            'none': None,
            'list': [1, 2, 3],
            'dict': {'a': 1, 'b': 2},
            'date': datetime.datetime(2023, 1, 1)
        }
        
        # Mock _prepare_context_dict to return the safe dict
        with patch.object(view_generator, '_prepare_context_dict', return_value=safe_dict):
            with patch.object(view_generator, '_prepare_metadata', return_value={}):
                with patch('django_spellbook.management.commands.processing.view_generator.repr', return_value=str(safe_dict)):
                    # Should not log any security errors
                    with patch('django_spellbook.management.commands.processing.view_generator.logger') as mock_logger:
                        result = view_generator._generate_view_function(processed_file)
                        
                        # Error should not be called for security validation
                        mock_logger.error.assert_not_called()
                        
                        # Should use the original context
                        self.assertNotIn("context = {}", result)